# SPDX-FileCopyrightText: 2020-2021 National University of Entre Rios (UNER)
#                         School of Engineering
#                         <saludpublica@ingenieria.uner.edu.ar>
# SPDX-FileCopyrightText: 2020 Mario Puntin <mario@silix.com.ar>
# SPDX-FileCopyrightText: 2020-2024 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2020-2024 GNU Solidario <health@gnusolidario.org>

# SPDX-License-Identifier: GPL-3.0-or-later
########################################################################
#   Hospital Management Information System (HMIS) component of the     #
#                       GNU Health project                             #
#                   https://www.gnuhealth.org                          #
########################################################################
#                         HEALTH DENTISTRY package                     #
#                odontogram_report.py: odontogram report               #
########################################################################
import io
import json
from PIL import Image, ImageDraw, ImageFont
from trytond.report import Report


__all__ = ['Odontogram']


class Odontogram(Report):
    __name__ = 'health_dentistry.odontogram.report'

    x_distance = 86
    y_distance = 110

    __pieces = {
        '18': (1, 1), '17': (2, 1), '16': (3, 1), '15': (4, 1),
        '14': (5, 1), '13': (6, 1), '12': (7, 1), '11': (8, 1),

        '21': (10, 1), '22': (11, 1), '23': (12, 1), '24': (13, 1),
        '25': (14, 1), '26': (15, 1), '27': (16, 1), '28': (17, 1),

        '48': (1, 4), '47': (2, 4), '46': (3, 4), '45': (4, 4),
        '44': (5, 4), '43': (6, 4), '42': (7, 4), '41': (8, 4),

        '31': (10, 4), '32': (11, 4), '33': (12, 4), '34': (13, 4),
        '35': (14, 4), '36': (15, 4), '37': (16, 4), '38': (17, 4),

        '55': (4, 2), '54': (5, 2), '53': (6, 2),
        '52': (7, 2), '51': (8, 2),

        '61': (10, 2), '62': (11, 2), '63': (12, 2),
        '64': (13, 2), '65': (14, 2),

        '85': (4, 3), '84': (5, 3), '83': (6, 3),
        '82': (7, 3), '81': (8, 3),

        '71': (10, 3), '72': (11, 3), '73': (12, 3),
        '74': (13, 3), '75': (14, 3),
    }

    pieces = {}
    for key, value in __pieces.items():
        pieces[key] = (x_distance / 2 + x_distance * (value[0] - 1),
                       x_distance / 2 + y_distance * (value[1] - 1))

    image_size = (x_distance * 17, y_distance * 4)

    @classmethod
    def plot_teeth(cls, image, dschema):
        draw = ImageDraw.Draw(image)

        for tooth, values in cls.pieces.items():
            width = 3
            color = (0, 0, 0)

            x = values[0]
            y = values[1]
            d1 = cls.x_distance / 2 * 0.9
            d2 = d1 / 2
            d3 = d1 / 1.414
            d4 = d2 / 1.414

            draw.ellipse((x - d1, y - d1, x + d1, y + d1),
                         outline=color, width=width)

            if tooth in dschema.keys():
                draw.ellipse(
                    (x - d2, y - d2, x + d2, y + d2),
                    outline=color,
                    width=width)

                draw.line((x - d3, y - d3, x - d4, y - d4),
                          fill=color, width=width)
                draw.line((x + d3, y + d3, x + d4, y + d4),
                          fill=color, width=width)
                draw.line((x - d3, y + d3, x - d4, y + d4),
                          fill=color, width=width)
                draw.line((x + d3, y - d3, x + d4, y - d4),
                          fill=color, width=width)

            fontsize = cls.x_distance // 4
            # Note: load_default support size argument when
            # pillow-10.1.0
            font = ImageFont.load_default(size=fontsize)
            draw.multiline_text(
                (x - fontsize * 0.65, y + d1 * 1.1),
                tooth,
                fill=color,
                font=font)

    @classmethod
    def plot_extraction(cls, image, piece_center, status):
        missing_color = "#0000ff"  # blue
        for_extraction_color = "#ff0000"  # red
        if (status == 'M'):
            color = missing_color
        else:
            color = for_extraction_color

        xcenter, ycenter = piece_center
        num = cls.x_distance / (2 * 1.414)
        llc = {'x': xcenter - num, 'y': ycenter + num}
        urc = {'x': xcenter + num, 'y': ycenter - num}
        ulc = {'x': xcenter - num, 'y': ycenter - num}
        lrc = {'x': xcenter + num, 'y': ycenter + num}
        draw = ImageDraw.Draw(image)
        draw.line((llc['x'], llc['y'], urc['x'], urc['y']),
                  fill=color, width=10)

        draw.line((ulc['x'], ulc['y'], lrc['x'], lrc['y']),
                  fill=color, width=10)

        return (image)

    @classmethod
    def plot_decayed(cls, image, tooth, piece_center, status):
        x, y = piece_center
        filling = (0, 0, 255)  # blue
        decayed = (255, 0, 0)  # red

        # Pick the color for decay of filling
        if status['ts'] == 'D':
            color = decayed
        else:
            color = filling

        position = (x, y)  # Center of the tooth
        ImageDraw.Draw(image)

        tregions = status.copy()
        # Delete ts element and focus on the tooth areas
        tregions.pop('ts')

        # Set the section of the filling / decay
        # Maxillar / upper region
        num = cls.x_distance / (2 * 1.414)

        if (tooth in range(11, 28) or tooth in range(51, 65)):
            for key in tregions.keys():
                if (key in ['o', 'i']):  # Occlusal or Incisal
                    position = (x, y)  # Center of the tooth
                if (key == 'v'):  # Vestibular
                    position = (x, y - num)
                if (key == 'p'):  # Palatine
                    position = (x, y + num)
                if (key == 'd'):  # Distal
                    position = (x - num, y)
                if (key == 'm'):  # Mesial
                    position = (x + num, y)

                ImageDraw.floodfill(
                    image, xy=position, value=color, thresh=200)

        # Mandibular / lower region
        if (tooth in range(31, 48) or tooth in range(71, 85)):
            for key in tregions.keys():
                if (key in ['o', 'i']):  # Occlusal or Incisal
                    position = (x, y)  # Center of the tooth
                if (key == 'l'):  # Lingual
                    position = (x, y - num)
                if (key == 'v'):  # Vestibular
                    position = (x, y + num)
                if (key == 'm'):  # Mesial
                    position = (x - num, y)
                if (key == 'd'):  # Distal
                    position = (x + num, y)

                ImageDraw.floodfill(
                    image, xy=position, value=color, thresh=200)

        return (image)

    @classmethod
    def plot_odontogram(cls, patient):

        image = Image.new('RGB', cls.image_size, (255, 255, 255))

        dschema1 = json.loads(patient.dental_schema or "{}")

        if patient.use_primary_schema:
            dschema2 = json.loads(patient.dental_schema_primary or "{}")
        else:
            dschema2 = {}

        dschema = {**dschema1, **dschema2}

        cls.plot_teeth(image, dschema)

        for tooth, values in dschema.items():
            # Decayed or filled tooth
            # Plot it first to avoid wrong surface filling if overlapping with
            # other symbols
            if (values['ts'] in ('D', 'F')):
                # Get all the keys (ts, o, m, d, ...)
                status = dschema[tooth]
                cls.plot_decayed(
                    image, int(tooth), cls.pieces[tooth], status)

            # Missing or set for extraction tooth
            if (values['ts'] in ('M', 'E')):
                status = values['ts']
                cls.plot_extraction(image, cls.pieces[tooth], status)

        holder = io.BytesIO()
        image.save(holder, 'png')
        image_png = holder.getvalue()
        holder.close()
        return (image_png)

    @classmethod
    def get_context(cls, records, header, data):
        context = super(Odontogram, cls).get_context(
            records, header, data)

        context['get_patient_odontogram'] = cls.plot_odontogram

        return context



# SPDX-FileCopyrightText:  2024 - Wei Zhao <wei.zhao@uclouvain.be>
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateTransition, StateView, Button
from trytond.pool import Pool
from trytond.exceptions import UserError
from httpx import ConnectError
from pyorthanc import Orthanc
import logging

logger = logging.getLogger(__name__)


__all__ = ["AddOrthancInitData", "ConnectNewOrthancServer"]


class AddOrthancInitData(ModelView):
    """Init data for Orthanc connection"""

    __name__ = "gnuhealth.imaging_orthanc.orthanc_add_init_data"

    label = fields.Char(
        "Label", required=True,
        help="The label of the Orthanc server. Must be unique"
    )
    domain = fields.Char(
        "URL", required=True,
        help="The full URL of the Orthanc server. Must be unique"
    )

    proxy_domain = fields.Char(
        "Proxy URL", required=False,
        help="Optional Proxy URL of the Orthanc server. "
        "It's only needed when the client "
        "cannot connect directly to the Orthanc server."
    )

    user = fields.Char(
        "Username", required=True,
        help="Username for Orthanc REST server"
    )
    password = fields.Char(
        "Password", required=True,
        help="Password for Orthanc REST server"
    )


class ConnectNewOrthancServer(Wizard):
    "Connect new Orthanc server"
    __name__ = "gnuhealth.imaging_orthanc.wizard.orthanc_new_connect"

    start = StateView(
        "gnuhealth.imaging_orthanc.orthanc_add_init_data",
        "health_orthanc.view_orthanc_add_init_data",
        [
            Button("Cancel", "end", "tryton-cancel"),
            Button("Connect", "connect", "tryton-ok", default=True),
        ]
    )

    connect = StateTransition()

    def transition_connect(self):
        pool = Pool()
        Config = pool.get("gnuhealth.orthanc.config")

        # check if domain already exists
        if len(Config.search([["label", "=", self.start.label]])) > 0 or len(
                Config.search([["domain", "=", self.start.domain]])) > 0:
            raise UserError(
                "There is already a server configuration",
                "with the same label or URL")
        else:
            try:
                client = Orthanc(
                    url=self.start.domain,
                    username=self.start.user,
                    password=self.start.password)
                # check if connection works
                client.get_changes({"limit": 1})
            except ConnectError as err:
                raise UserError(
                    "Cannot connect to the specified Orthanc server.",
                    "Is the URL correct?" + str(err))
            except Exception as err:
                logger.error(type(err))
                raise UserError(
                    "Configure Connection Error: " +
                    "Please ensure that the URL provided is " +
                    "the correct Orthanc server URL. " +
                    "To verify, try accessing the URL in a web browser. " +
                    str(err))
            else:
                new_server = {
                    "label": self.start.label,
                    "domain": self.start.domain,
                    "proxy_domain": self.start.proxy_domain,
                    "user": self.start.user,
                    "password": self.start.password,
                }
                server, = Config.create([new_server])
                server.validated = True
                Config.save([server])
        return 'end'

    def end(self):
        return 'reload'



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################

from datetime import date
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, StateAction, Button
from dateutil.relativedelta import relativedelta

__all__ = ['EpidemicsReportStart', 'EpidemicsReport']


class EpidemicsReportStart(ModelView):
    'Epidemics Report Start'
    __name__ = 'gnuhealth.epidemics.report.open.start'

    health_condition = fields.Many2One(
        'gnuhealth.pathology', 'Condition', required=True)

    start_date = fields.Date("Start")
    end_date = fields.Date("End")

    demographics = fields.Boolean("Demographics")

    @staticmethod
    def default_start_date():
        return date.today() - relativedelta(days=7)

    @staticmethod
    def default_end_date():
        return date.today()

    @staticmethod
    def default_demographics():
        return True


class EpidemicsReport(Wizard):
    'Open Institution Epidemics Report'
    __name__ = 'gnuhealth.epidemics.report.open'

    start = StateView(
        'gnuhealth.epidemics.report.open.start',
        'health_reporting.epidemics_report_open_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Open', 'open_', 'tryton-ok', default=True),
        ])

    open_ = StateAction('health_reporting.report_epidemics_information')

    def fill_data(self):
        return {
            'health_condition': (
                self.start.health_condition.id
                if self.start.health_condition else None),
            'start_date': self.start.start_date,
            'end_date': self.start.end_date,
            'demographics': self.start.demographics,
        }

    def do_open_(self, action):
        return action, self.fill_data()

    def transition_open_(self):
        return 'end'



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                      HEALTH SURGERY package                           #
#               __init__.py: Package declaration file                   #
#########################################################################

from trytond.pool import Pool
from . import health_surgery
from . import health
from . import report
from . import sequences


def register():
    Pool.register(
        sequences.GnuHealthSequences,
        sequences.SurgeryCodeSequence,
        health_surgery.RCRI,
        health_surgery.Surgery,
        health_surgery.Operation,
        health.PatientProcedure,
        health_surgery.SurgerySupply,
        health_surgery.PatientData,
        health_surgery.SurgeryTeam,
        health_surgery.SurgeryComplication,
        health_surgery.PreOperativeAssessment,
        health_surgery.SurgeryProtocol,
        health_surgery.SurgeryDrain,
        health_surgery.PatientEvaluation,
        health_surgery.ORScheduler,
        module='health_surgery', type_='model')
    Pool.register(
        report.surgery_report.SurgeryReport,
        module='health_surgery', type_='report')



# SPDX-FileCopyrightText: 2017-2024 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2017-2024 Luis Falcon <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.model.exceptions import ValidationError


class InvalidAttachmentName(ValidationError):
    pass



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                HEALTH INPATIENT CALENDAR package                      #
#         test_health_inpatient_calendar.py main test module            #
#########################################################################
import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class HealthInpatientCalendarTestCase(ModuleTestCase):
    '''
    Test Health Inpatient Calendar module.
    '''
    module = 'health_inpatient_calendar'


del ModuleTestCase


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        HealthInpatientCalendarTestCase))
    return suite



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                       HEALTH GYNECO package                           #
#              __init__.py: Package declaration file                    #
#########################################################################
from trytond.pool import Pool
from . import health_gyneco


def register():
    Pool.register(
        health_gyneco.PatientPregnancy,
        health_gyneco.PregnancyResult,
        health_gyneco.PrenatalEvaluation,
        health_gyneco.PuerperiumMonitor,
        health_gyneco.Perinatal,
        health_gyneco.PerinatalMonitor,
        health_gyneco.GnuHealthPatient,
        health_gyneco.PatientMenstrualHistory,
        health_gyneco.PatientMammographyHistory,
        health_gyneco.PatientPAPHistory,
        health_gyneco.PatientColposcopyHistory,
        module='health_gyneco', type_='model')



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                           HEALTH package                              #
#                __init__.py: Package declaration file                  #
#########################################################################

from trytond.pool import Pool
from trytond.report import Report
from . import health
from . import sequences
from . import wizard
from . import report
from . import country


def register():
    Pool.register(
        country.Subdivision,
        health.OperationalArea,
        health.OperationalSector,
        health.DomiciliaryUnit,
        health.FederationCountryConfig,
        health.Occupation,
        health.Ethnicity,
        health.BirthCertificate,
        health.DeathCertificate,
        health.Party,
        health.ContactMechanism,
        health.PersonName,
        health.PartyAddress,
        health.DrugDoseUnits,
        health.MedicationFrequency,
        health.DrugForm,
        health.DrugRoute,
        health.MedicalSpecialty,
        health.HealthInstitution,
        health.HealthInstitutionSpecialties,
        health.HealthInstitutionOperationalSector,
        health.HealthInstitutionO2M,
        health.HospitalBuilding,
        health.HospitalUnit,
        health.HospitalOR,
        health.HospitalWard,
        health.HospitalBed,
        health.HealthProfessional,
        health.HealthProfessionalSpecialties,
        health.Family,
        health.FamilyMember,
        health.FamilyDiseases,
        health.MedicamentCategory,
        health.Medicament,
        health.ImmunizationSchedule,
        health.ImmunizationScheduleLine,
        health.ImmunizationScheduleDose,
        health.PathologyCategory,
        health.PathologyGroup,
        health.Pathology,
        health.DiseaseMembers,
        health.BirthCertExtraInfo,
        health.DeathCertExtraInfo,
        health.DeathUnderlyingCondition,
        health.ProcedureCode,
        health.InsurancePlan,
        health.Insurance,
        health.AlternativePersonID,
        health.Product,
        health.PatientData,
        health.PatientDiseaseInfo,
        health.Appointment,
        health.AppointmentReport,
        health.OpenAppointmentReportStart,
        health.PatientPrescriptionOrder,
        health.PrescriptionLine,
        health.PatientMedication,
        health.PatientVaccination,
        health.PatientProcedure,
        health.PatientEvaluation,
        health.Directions,
        health.SecondaryCondition,
        health.DiagnosticHypothesis,
        health.SignsAndSymptoms,
        health.PatientECG,
        health.ProductTemplate,
        health.PageOfLife,
        health.ProceduresConfig,
        health.Commands,
        health.Modules,
        health.Help,
        health.OnlineDocument,
        wizard.wizard_check_immunization_status.CheckImmunizationStatusInit,
        sequences.GnuHealthSequences,
        sequences.PatientSequence,
        sequences.PatientEvaluationSequence,
        sequences.AppointmentSequence,
        sequences.PrescriptionSequence,
        module='health', type_='model')

    Pool.register(
        health.OpenAppointmentReport,
        wizard.wizard_update_patient_disease_info.UpdatePatientDiseaseInfo,
        wizard.wizard_appointment_evaluation.CreateAppointmentEvaluation,
        wizard.wizard_check_immunization_status.CheckImmunizationStatus,
        module='health', type_='wizard')

    Pool.register(
        report.immunization_status_report.ImmunizationStatusReport,
        module='health', type_='report')

    Pool.register_mixin(
        report.health_report.ReportDateAndTimeMixin, Report,
        module='health')

    Pool.register_mixin(
        report.health_report.ReportImageToolMixin, Report,
        module='health')

    Pool.register_mixin(
        report.health_report.ReportGettextMixin, Report,
        module='health')



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>

# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                    HEALTH LIFESTYLE package                           #
#                 health_lifestyle.py: main module                      #
#########################################################################
from trytond.model import ModelView, ModelSQL, fields, Unique
from datetime import datetime
from trytond.pool import PoolMeta

__all__ = ['VegetarianTypes', 'DietBelief', 'DrugsRecreational',
           'PatientRecreationalDrugs', 'PatientCAGE', 'MedicalPatient']


class VegetarianTypes(ModelSQL, ModelView):
    'Vegetarian Types'
    __name__ = 'gnuhealth.vegetarian_types'

    name = fields.Char(
        'Vegetarian', translate=True, required=True,
        help="Vegetarian")
    code = fields.Char(
        'Code', required=True,
        help="Short description")
    desc = fields.Char(
        'Description', translate=True, required=True,
        help="Short description")


# Diet by belief / religion
class DietBelief (ModelSQL, ModelView):
    'Diet by Belief'
    __name__ = "gnuhealth.diet.belief"

    name = fields.Char('Belief', required=True, translate=True)
    code = fields.Char('Code', required=True)
    description = fields.Text('Description', required=True, translate=True)

    @classmethod
    def __setup__(cls):
        super(DietBelief, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('code_unique', Unique(t, t.code),
                'The Diet code already exists'),
        ]


class DrugsRecreational(ModelSQL, ModelView):
    'Recreational Drug'
    __name__ = 'gnuhealth.drugs_recreational'

    name = fields.Char(
        'Name', translate=True, required=True,
        help="Name of the drug")
    code = fields.Char(
        'Code', required=True,
        help="Please use CAPITAL LETTERS and no spaces")
    street_name = fields.Char(
        'Street names',
        help="Common name of the drug in street jargon")

    toxicity = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Low'),
        ('2', 'High'),
        ('3', 'Extreme'),
    ], 'Toxicity', sort=False)

    addiction_level = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Low'),
        ('2', 'High'),
        ('3', 'Extreme'),
    ], 'Dependence', sort=False)

    legal_status = fields.Selection([
        (None, ''),
        ('0', 'Legal'),
        ('1', 'Illegal'),
    ], 'Legal Status', sort=False)

    category = fields.Selection([
        (None, ''),
        ('cannabinoid', 'Cannabinoids'),
        ('depressant', 'Depressants'),
        ('dissociative', 'Dissociative Anesthetics'),
        ('hallucinogen', 'Hallucinogens'),
        ('opioid', 'Opioids'),
        ('stimulant', 'Stimulants'),
        ('other', 'Others'),
    ], 'Category', sort=False)

    withdrawal_level = fields.Integer(
        'Withdrawal',
        help="Presence and severity of characteristic withdrawal "
        "symptoms.\nUsing Henningfield rating. 1=highest and 6=lowest")

    reinforcement_level = fields.Integer(
        'Reinforcement',
        help="A measure of the substance's ability to get users to take it "
        " again and again, and in preference to other substances.\nUsing "
        " Henningfield rating. 1=highest and 6=lowest")

    tolerance_level = fields.Integer(
        'Tolerance',
        help="How much of the substance is needed to satisfy increasing "
        "cravings for it, and the level of stable need that is eventually "
        "reached.\nUsing Henningfield rating. 1=highest and 6=lowest")

    dependence_level = fields.Integer(
        'Dependence',
        help="How difficult it is for the user to quit, the relapse rate, "
        "the percentage of people who eventually become dependent, the "
        "rating users give their own need for the substance and the "
        "degree to which the substance will be used in the face of "
        "evidence that it causes harm.\nUsing Henningfield rating. "
        "1=highest and 6=lowest")

    intoxication_level = fields.Integer(
        'Intoxication',
        help="the level of intoxication is associated with addiction and "
        "increases the personal and social damage a substance may do. \n"
        "Using Henningfield rating. 1=highest and 6=lowest")

    route_oral = fields.Boolean('Oral')

    route_popping = fields.Boolean(
        'Skin Popping',
        help="Subcutaneous or Intradermical administration")

    route_inhaling = fields.Boolean(
        'Smoke / Inhale',
        help="Insufflation, excluding nasal")

    route_sniffing = fields.Boolean(
        'Sniffing',
        help="Also called snorting - inhaling through the nares")

    route_injection = fields.Boolean(
        'Injection',
        help="Injection - Intravenous, Intramuscular...")

    dea_schedule_i = fields.Boolean(
        'DEA schedule I',
        help="Schedule I and II drugs have a high potential for abuse. "
        "They require greater storage security and have a quota on "
        "manufacturing, among other restrictions. Schedule I drugs are "
        "available for research only and have no approved medical use; "
        "Schedule II drugs are available only by prescription "
        "(unrefillable) and require a form for ordering. Schedule III "
        "and IV drugs are available by prescription, may have five "
        "refills in 6 months, and may be ordered orally. "
        "Some Schedule V drugs are available over the counter")

    dea_schedule_ii = fields.Boolean(
        'II',
        help="Schedule I and II drugs have a high potential for abuse."
        "They require greater storage security and have a quota on"
        "manufacturing, among other restrictions. Schedule I drugs are"
        "available for research only and have no approved medical use; "
        "Schedule II drugs are available only by prescription "
        "(unrefillable) and require a form for ordering. Schedule III "
        "and IV drugs are available by prescription, may have five"
        "refills in 6 months, and may be ordered orally. "
        "Some Schedule V drugs are available over the counter")

    dea_schedule_iii = fields.Boolean(
        'III',
        help="Schedule I and II drugs have a high potential for abuse. "
        "They require greater storage security and have a quota on "
        "manufacturing, among other restrictions. Schedule I drugs are "
        "available for research only and have no approved medical use; "
        "Schedule II drugs are available only by prescription "
        "(unrefillable) and require a form for ordering. Schedule III "
        "and IV drugs are available by prescription, may have five "
        "refills in 6 months, and may be ordered orally. "
        "Some Schedule V drugs are available over the counter")

    dea_schedule_iv = fields.Boolean(
        'IV',
        help="Schedule I and II drugs have a high potential for abuse. "
        "They require greater storage security and have a quota on "
        "manufacturing, among other restrictions. Schedule I drugs are "
        "available for research only and have no approved medical use; "
        "Schedule II drugs are available only by prescription "
        "(unrefillable) and require a form for ordering. Schedule III "
        "and IV drugs are available by prescription, may have five "
        "refills in 6 months, and may be ordered orally. "
        "Some Schedule V drugs are available over the counter")

    dea_schedule_v = fields.Boolean(
        'V',
        help="Schedule I and II drugs have a high potential for abuse. "
        "They require greater storage security and have a quota on "
        "manufacturing, among other restrictions. Schedule I drugs are "
        "available for research only and have no approved medical use; "
        "Schedule II drugs are available only by prescription "
        "(unrefillable) and require a form for ordering. Schedule III "
        "and IV drugs are available by prescription, may have five "
        "refills in 6 months, and may be ordered orally. "
        "Some Schedule V drugs are available over the counter")

    info = fields.Text('Extra Info', translate=True)

    @classmethod
    def __setup__(cls):
        super(DrugsRecreational, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('name_unique', Unique(t, t.name),
                'The Recreational Drug name must be unique'),
            ('code_unique', Unique(t, t.code),
                'The Recreational Drug CODE must be unique'),
        ]


class PatientRecreationalDrugs(ModelSQL, ModelView):
    'Patient use of Recreational Drugs'
    __name__ = 'gnuhealth.patient.recreational_drugs'

    patient = fields.Many2One('gnuhealth.patient', 'Patient')
    recreational_drug = fields.Many2One(
        'gnuhealth.drugs_recreational',
        'Recreational Drug')


# CAGE questionnaire to assess patient dependency to alcohol
class PatientCAGE(ModelSQL, ModelView):
    'Patient CAGE Questionnaire'
    __name__ = 'gnuhealth.patient.cage'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)

    evaluation_date = fields.DateTime('Date')

    cage_c = fields.Boolean(
        'Hard to Cut down', help='Have you ever felt you '
        'needed to Cut down on your drinking?')
    cage_a = fields.Boolean(
        'Angry with Critics', help='Have people Annoyed '
        'you by criticizing your drinking?')
    cage_g = fields.Boolean(
        'Guilt', help='Have you ever felt Guilty about '
        'drinking?')
    cage_e = fields.Boolean(
        'Eye-opener', help='Have you ever felt you '
        'needed a drink first thing in the morning (Eye-opener) to steady '
        'your nerves or to get rid of a hangover?')

    cage_score = fields.Integer('CAGE Score')

    # Show the icon depending on the CAGE Score
    cage_warning_icon = \
        fields.Function(
            fields.Char('Cage Warning Icon'),
            'get_cage_warning_icon')

    def get_cage_warning_icon(self, name):
        if (self.cage_score > 1):
            return 'gnuhealth-warning'

    @fields.depends('cage_c', 'cage_a', 'cage_g', 'cage_e')
    def on_change_with_cage_score(self):
        total = 0

        if self.cage_c:
            total = total + 1
        if self.cage_a:
            total = total + 1
        if self.cage_g:
            total = total + 1
        if self.cage_e:
            total = total + 1

        return total

    @staticmethod
    def default_evaluation_date():
        return datetime.now()

    @staticmethod
    def default_cage_score():
        return 0

    @classmethod
    def __setup__(cls):
        super(PatientCAGE, cls).__setup__()

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None


class MedicalPatient(metaclass=PoolMeta):
    __name__ = 'gnuhealth.patient'

    exercise = fields.Boolean('Exercise')
    exercise_minutes_day = fields.Integer(
        'Minutes / day',
        help="How many minutes a day the patient exercises")
    sleep_hours = fields.Integer(
        'Hours of sleep',
        help="Average hours of sleep per day")
    sleep_during_daytime = fields.Boolean(
        'Sleeps at daytime',
        help="Check if the patient sleep hours are during daylight rather "
        "than at night")
    number_of_meals = fields.Integer('Meals per day')
    vegetarian_type = fields.Many2One(
        'gnuhealth.vegetarian_types', 'Vegetarian')
    diet_belief = fields.Many2One(
        'gnuhealth.diet.belief',
        'Belief', help="Enter the patient belief or religion")

    eats_alone = fields.Boolean(
        'Eats alone',
        help="Check this box if the patient eats by him / herself.")
    salt = fields.Boolean(
        'Salt',
        help="Check if patient consumes salt with the food")
    coffee = fields.Boolean('Coffee')
    coffee_cups = fields.Integer(
        'Cups per day',
        help="Number of cup of coffee a day")
    soft_drinks = fields.Boolean(
        'Soft drinks (sugar)',
        help="Check if the patient consumes soft drinks with sugar")
    diet = fields.Boolean(
        'Currently on a diet',
        help="Check if the patient is currently on a diet")
    diet_info = fields.Char(
        'Diet info',
        help="Short description on the diet")
    smoking = fields.Boolean('Smokes')
    smoking_number = fields.Integer('Cigarretes a day')
    ex_smoker = fields.Boolean('Ex-smoker')
    second_hand_smoker = fields.Boolean(
        'Passive smoker',
        help="Check it the patient is a passive / second-hand smoker")
    age_start_smoking = fields.Integer('Age started to smoke')
    age_quit_smoking = fields.Integer(
        'Age of quitting',
        help="Age of quitting smoking")
    alcohol = fields.Boolean('Drinks Alcohol')
    age_start_drinking = fields.Integer(
        'Age started to drink',
        help="Date to start drinking")
    age_quit_drinking = fields.Integer(
        'Age quit drinking',
        help="Date to stop drinking")
    ex_alcoholic = fields.Boolean('Ex alcoholic')
    alcohol_beer_number = fields.Integer('Beer / day')
    alcohol_wine_number = fields.Integer('Wine / day')
    alcohol_liquor_number = fields.Integer('Liquor / day')
    drug_usage = fields.Boolean('Drug Habits')
    ex_drug_addict = fields.Boolean('Ex drug addict')
    drug_iv = fields.Boolean(
        'IV drug user',
        help="Check this option if the patient injects drugs")
    age_start_drugs = fields.Integer(
        'Age started drugs',
        help="Age of start drugs")
    age_quit_drugs = fields.Integer(
        'Age quit drugs',
        help="Date of quitting drugs")
    recreational_drugs = fields.One2Many(
        'gnuhealth.patient.recreational_drugs', 'patient', 'Drugs')
    traffic_laws = fields.Boolean(
        'Obeys Traffic Laws',
        help="Check if the patient is a safe driver")
    car_revision = fields.Boolean(
        'Car Revision',
        help="Maintain the vehicle. Do periodical checks - tires,breaks ...")
    car_seat_belt = fields.Boolean(
        'Seat belt',
        help="Safety measures when driving : safety belt")
    car_child_safety = fields.Boolean(
        'Car Child Safety',
        help="Safety measures when driving : child seats, "
        "proper seat belting, not seating on the front seat, ....")
    home_safety = fields.Boolean(
        'Home safety',
        help="Keep safety measures for kids in the kitchen, "
        "correct storage of chemicals, ...")
    motorcycle_rider = fields.Boolean(
        'Motorcycle Rider',
        help="The patient rides motorcycles")
    helmet = fields.Boolean(
        'Uses helmet',
        help="The patient uses the proper motorcycle helmet")

    lifestyle_info = fields.Text('Extra Information')

    sexual_preferences = fields.Selection([
        (None, ''),
        ('h', 'Heterosexual'),
        ('g', 'Homosexual'),
        ('b', 'Bisexual'),
        ('t', 'Transexual'),
    ], 'Sexual Preferences', sort=False)

    sexual_preferences_str = sexual_preferences.translated(
        'sexual_preferences')

    sexual_practices = fields.Selection([
        (None, ''),
        ('s', 'Safe / Protected sex'),
        ('r', 'Risky / Unprotected sex'),
    ], 'Sexual Practices', sort=False)

    sexual_practices_str = sexual_practices.translated('sexual_practices')

    sexual_partners = fields.Selection([
        (None, ''),
        ('m', 'Monogamous'),
        ('t', 'Polygamous'),
    ], 'Sexual Partners', sort=False)

    sexual_partners_str = sexual_partners.translated('sexual_partners')

    sexual_partners_number = fields.Integer('Number of sexual partners')

    first_sexual_encounter = fields.Integer('Age first sexual encounter')

    anticonceptive = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Pill / Minipill'),
        ('2', 'Male condom'),
        ('3', 'Vasectomy'),
        ('4', 'Female sterilisation'),
        ('5', 'Intra-uterine device'),
        ('6', 'Withdrawal method'),
        ('7', 'Fertility cycle awareness'),
        ('8', 'Contraceptive injection'),
        ('9', 'Skin Patch'),
        ('10', 'Female condom'),
    ], 'Contraceptive Method', sort=False)

    sex_oral = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Active'),
        ('2', 'Passive'),
        ('3', 'Both'),
    ], 'Oral Sex', sort=False)

    sex_anal = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Active'),
        ('2', 'Passive'),
        ('3', 'Both'),
    ], 'Anal Sex', sort=False)

    prostitute = fields.Boolean(
        'Prostitute',
        help="Check if the patient (he or she) is a prostitute")
    sex_with_prostitutes = fields.Boolean(
        'Sex with prostitutes',
        help="Check if the patient (he or she) has sex with prostitutes")

    sexuality_info = fields.Text('Extra Information')

    cage = fields.One2Many(
        'gnuhealth.patient.cage',
        'patient', 'CAGE')



# SPDX-FileCopyrightText:  2024 - Wei Zhao <wei.zhao@uclouvain.be>
# SPDX-License-Identifier: GPL-3.0-or-later

from pyorthanc import Orthanc
import logging
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, StateTransition, Button
from trytond.exceptions import UserError
from trytond.pool import Pool

from io import BytesIO

__all__ = ['UploadImageDataStart', 'UploadImageData']

logger = logging.getLogger(__name__)

#
# Start upload image data view
#


class UploadImageDataStart(ModelView):
    "Upload Image Data Start"
    __name__ = "gnuhealth.imaging_orthanc.upload_image_data.start"
    # The image data needs to be uploaded to the orthanc server.
    data_to_upload = fields.Binary("File to upload", required=True)
    # The target orthanc server where the image data will be saved
    server_config = fields.Many2One(
        'gnuhealth.orthanc.config',
        'Server',
        help='Orthanc server',
        required=True)

#
# Uploading of image data
#


class UploadImageData(Wizard):
    'Upload Image Data'
    __name__ = 'gnuhealth.imaging_orthanc.upload_image_data'
    start = StateView(
        'gnuhealth.imaging_orthanc.upload_image_data.start',
        'health_orthanc.upload_image_data_start_form',
        [Button('Cancel', 'end', 'tryton-cancel'),
         Button('Upload Image Data',
                'upload', 'tryton-ok',
                validate=True)])
    upload = StateTransition()

    def upload_image_data(self, data_to_upload, server_config):
        # A function to upload image data to the selected server.
        # data_to_upload contains the byte data of multiple dicom files.
        # The format is:
        # 1.the bytes 'M', 'U', 'L', 'T' (ASCII 77,85,76,84)
        # 2.8 bytes containing the length of the file (Little Endian)
        # 3.the data of the file
        # 4.repeat steps 2 and 3 for next file
        try:
            if (data_to_upload[0] == 77
                and data_to_upload[1] == 85
                and data_to_upload[2] == 76
                    and data_to_upload[3] == 84):
                pos = 4
                while pos < len(data_to_upload):
                    # get length of file from data
                    data_length = 0

                    for i in range(0, 8):
                        data_length = (
                            data_to_upload[pos + 7 - i]
                            + data_length
                            * 256)

                    pos = pos + 8
                    # get content of file from data
                    data = BytesIO(data_to_upload[pos: pos + data_length])
                    pos = pos + data_length
                    # send file to Orthanc
                    client = Orthanc(
                        url=server_config.domain,
                        username=server_config.user,
                        password=server_config.password,
                        timeout=600)
                    client.post_instances(data.getvalue())
            else:
                data = BytesIO(data_to_upload[4:])
                client = Orthanc(
                    url=server_config.domain,
                    username=server_config.user,
                    password=server_config.password,
                    timeout=600)
                client.post_instances(data.getvalue())
        except Exception as exception:
            logger.error('Upload dicom files: %s', exception, exc_info=True)
            raise UserError(str(exception))

    def transition_upload(self):
        # Transitions the upload process by uploading the image data specified
        # in the start view

        self.upload_image_data(
            self.start.data_to_upload,
            self.start.server_config)  # noqa	E501

        Pool().get('gnuhealth.imaging_orthanc.study').get_new_studies()
        return 'end'

    def end(self):
        return 'reload'



# SPDX-FileCopyrightText: 2019-2022 Chris Zimmerman <chris@teffalump.com>
# SPDX-FileCopyrightText: 2021-2024 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2021-2024 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                       HEALTH ORTHANC package                          #
#                  __init__.py: Package declaration file                #
#########################################################################

"""
Initialization module for the ``health_orthanc`` module.

This module registers the necessary classes and methods of the
``health_orthanc`` module and its wizard in the Tryton pool.
This allows other modules to access the functionalities provided by
the ``health_orthanc`` module.
"""

from trytond.pool import Pool
from . import health_orthanc
from . import health_orthanc_configuration
from . import wizard


def register():
    Pool.register(
        health_orthanc.View,
        health_orthanc.Patient,
        health_orthanc.TestResult,
        health_orthanc.PatientOrthancStudy,
        health_orthanc.StudySeries,
        health_orthanc.SeriesInstances,
        health_orthanc_configuration.ServerConfig,
        wizard.wizard_upload_image_data.UploadImageDataStart,
        wizard.wizard_get_new_studies.GetNewStudiesStart,
        wizard.wizard_full_synchronize.FullSynchronizeStart,
        wizard.wizard_orthanc_config.AddOrthancInitData,
        # DEPRECATED, Used to migrate date.
        health_orthanc.OrthancPatientDEPRECATED,
        health_orthanc.OrthancStudyDEPRECATED,
        module="health_orthanc",
        type_="model",
    )

    Pool.register(
        wizard.wizard_full_synchronize.FullSynchronize,
        wizard.wizard_get_new_studies.GetNewStudies,
        wizard.wizard_upload_image_data.UploadImageData,
        wizard.wizard_orthanc_config.ConnectNewOrthancServer,
        module='health_orthanc', type_='wizard'
    )



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                      HEALTH INSURANCE package                         #
#                  health_insurance.py: main module                     #
#########################################################################

from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval
from trytond.i18n import gettext
from trytond.pool import PoolMeta
import decimal

from .exceptions import (DiscountPctOutOfRange, NeedAPolicy,
                         DiscountWithoutElement)


__all__ = ['InsurancePlanProductPolicy', 'InsurancePlan', 'HealthService',
           'PatientProcedure']


class InsurancePlanProductPolicy(ModelSQL, ModelView):
    'Policy associated to the product on the insurance plan'
    __name__ = "gnuhealth.insurance.plan.product.policy"

    plan = fields.Many2One(
        'gnuhealth.insurance.plan',
        'Plan', required=True)

    product = fields.Many2One('product.product', 'Product')

    icode = fields.Char(
        'Product code',
        help="Equivalent product code for this plan"
    )

    product_category = fields.Many2One('product.category', 'Category')

    discount = fields.Float(
        'Discount', digits=(3, 2),
        help="Discount in Percentage. It has higher precedence than "
             "the fixed price when both values coexist")

    price = fields.Float(
        'Price', help="Apply a fixed price for this product or category")

    @classmethod
    def validate(cls, policies):
        super(InsurancePlanProductPolicy, cls).validate(policies)
        for policy in policies:
            policy.validate_discount()
            policy.validate_policy_elements()

    def validate_discount(self):
        if (self.discount):
            if (self.discount < 0 or self.discount > 100):
                raise DiscountPctOutOfRange(
                    gettext('health_insurance.msg_pct_out_of_range')
                )
        if (not self.discount and not self.price):
            raise NeedAPolicy(
                gettext('health_insurance.msg_need_a_policy')
            )

    def validate_policy_elements(self):
        if (not self.product and not self.product_category):
            raise DiscountWithoutElement(
                gettext('health_insurance.msg_discount_without_element')
            )


class InsurancePlan(metaclass=PoolMeta):
    __name__ = "gnuhealth.insurance.plan"

    product_policy = fields.One2Many(
        'gnuhealth.insurance.plan.product.policy',
        'plan', 'Policy')


class HealthService(metaclass=PoolMeta):
    __name__ = 'gnuhealth.health_service'

    insurance_holder = fields.Many2One(
        'party.party', 'Insurance of',
        help="Insurance Policy Holder")

    insurance_plan = fields.Many2One(
        'gnuhealth.insurance',
        'Plan',
        domain=[('party', '=', Eval('insurance_holder'))],
        depends=['insurance_holder'])

    # Set the insurance holder upon entering the patient
    @fields.depends('patient')
    def on_change_patient(self):
        if self.patient:
            self.insurance_holder = self.patient.party


class PatientProcedure(metaclass=PoolMeta):
    __name__ = 'gnuhealth.patient.procedure'

    price = fields.Numeric('Price')

    """ Set the default price on the procedure
        based on the patient insurance plan product counterpart
    """
    @fields.depends('procedure', 'insurance')
    def on_change_with_price(self):
        if (self.procedure):
            if self.procedure.product:
                prd = self.procedure.product
                if self.insurance:
                    for pline in self.insurance.plan_id.product_policy:
                        if pline.product == prd:
                            return decimal.Decimal(pline.price)



# SPDX-FileCopyrightText: 2009-2013 Bertrand Chenal
# SPDX-FileCopyrightText: 2009-2016 B2CK
# SPDX-FileCopyrightText: 2009-2016 Cédric Krier
# SPDX-FileCopyrightText: 2016-2024 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2016-2024 Luis Falcón <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.model.exceptions import ValidationError


class InvalidCalendarExtension(ValidationError):
    pass


class InvalidRecurrence(ValidationError):
    pass


class InvalidBySecond(ValidationError):
    pass


class InvalidByMinute(ValidationError):
    pass


class InvalidByHour(ValidationError):
    pass


class InvalidByDay(ValidationError):
    pass


class InvalidByMonthDay(ValidationError):
    pass


class InvalidByWeekNumber(ValidationError):
    pass


class InvalidByYearDay(ValidationError):
    pass


class InvalidByMonth(ValidationError):
    pass


class InvalidBySetPosition(ValidationError):
    pass



# SPDX-FileCopyrightText: 2012-2017 Cédric Krier
# SPDX-FileCopyrightText: 2017-2024 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2017-2024 Luis Falcon <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class WebdavTestCase(ModuleTestCase):
    'Test Webdav module'
    module = 'health_webdav3_server'


del ModuleTestCase


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        WebdavTestCase))
    return suite



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from . import wizard_create_prescription_stock_move
from . import wizard_create_vaccination_stock_move


__all__ = ['wizard_create_prescription_stock_move',
           'wizard_create_vaccination_stock_move']



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                       HEALTH DISABILITY package                       #
#                   health_disability.py: main module                   #
#########################################################################
# THIS PACKAGE APPLIES SOME CONCEPTS FROMTHE WORLD HEALTH ORGANIZATION  #
# INTERNATIONAL CLASSIFICATION OF FUNCTIONING, DISABILITY AND HEALTH    #
#         REFER TO THE DOCUMENTATION FOR FURTHER INFORMATION            #
#########################################################################

from datetime import date
from trytond.model import ModelView, ModelSQL, fields, Unique
from trytond.modules.health.core import get_health_professional

__all__ = ['GnuHealthPatient',
           'Product', 'BodyFunctionCategory', 'BodyFunction',
           'BodyStructureCategory', 'BodyStructure',
           'ActivityAndParticipationCategory', 'ActivityAndParticipation',
           'EnvironmentalFactorCategory', 'EnvironmentalFactor',
           'PatientDisabilityAssessment',
           'PatientBodyFunctionAssessment',
           'PatientBodyStructureAssessment',
           'PatientActivityAndParticipationAsssessment',
           'PatientEnvironmentalFactorAssessment',
           'PatientAmputation', 'PatientProthesis',
           'PatientData']


# Include disabilty amputation and UXO casualty information on patient model
class GnuHealthPatient(ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient'

    disability = fields.Boolean(
        'Disabilities / Barriers', help="Mark this "
        "box if the patient has history of significant disabilities and/or "
        "barriers. Review the Disability Assessments, socioeconomic info,  "
        "diseases and surgeries for more details")
    uxo = fields.Boolean('UXO', help="UXO casualty")
    amputee = fields.Boolean(
        'Amputee', help="Person has had one or more"
        " limbs removed by amputation. Includes congenital conditions")
    amputee_since = fields.Date('Since', help="Date of amputee status")

    amputations = fields.One2Many(
        'gnuhealth.patient.amputation', 'patient', 'Amputations')

    protheses = fields.One2Many(
        'gnuhealth.patient.prothesis', 'patient', 'Protheses')


class Product(ModelSQL, ModelView):
    'Product'
    __name__ = 'product.product'

    is_prothesis = fields.Boolean(
        'Prothesis', help='Check if the product is a prothesis')


class BodyFunctionCategory(ModelSQL, ModelView):
    'Body Function Category'
    __name__ = 'gnuhealth.body_function.category'

    name = fields.Char('Name', required=True, translate=True)
    code = fields.Char('code', required=True)

    @classmethod
    def __setup__(cls):
        super(BodyFunctionCategory, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class BodyFunction(ModelSQL, ModelView):
    'Body Functions'
    __name__ = 'gnuhealth.body_function'

    name = fields.Char('Function', required=True, translate=True)
    code = fields.Char('code', required=True)
    category = fields.Many2One('gnuhealth.body_function.category', 'Category')

    @classmethod
    def __setup__(cls):
        super(BodyFunction, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class BodyStructureCategory(ModelSQL, ModelView):
    'Body Structure Category'
    __name__ = 'gnuhealth.body_structure.category'

    name = fields.Char('Name', required=True, translate=True)
    code = fields.Char('code', required=True)

    @classmethod
    def __setup__(cls):
        super(BodyStructureCategory, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class BodyStructure(ModelSQL, ModelView):
    'Body Functions'
    __name__ = 'gnuhealth.body_structure'

    name = fields.Char('Structure', required=True, translate=True)
    code = fields.Char('code', required=True)
    category = fields.Many2One('gnuhealth.body_structure.category', 'Category')

    @classmethod
    def __setup__(cls):
        super(BodyStructure, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class ActivityAndParticipationCategory(ModelSQL, ModelView):
    'Activity and Participation Category'
    __name__ = 'gnuhealth.activity_and_participation.category'

    name = fields.Char('Name', required=True, translate=True)
    code = fields.Char('code', required=True)

    @classmethod
    def __setup__(cls):
        super(ActivityAndParticipationCategory, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class ActivityAndParticipation(ModelSQL, ModelView):
    'Activity limitations and participation restrictions'
    __name__ = 'gnuhealth.activity_and_participation'

    name = fields.Char('A & P', required=True, translate=True)
    code = fields.Char('code', required=True)
    category = fields.Many2One(
        'gnuhealth.activity_and_participation.category', 'Category')

    @classmethod
    def __setup__(cls):
        super(ActivityAndParticipation, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class EnvironmentalFactorCategory(ModelSQL, ModelView):
    'Environmental Factor Category'
    __name__ = 'gnuhealth.environmental_factor.category'

    name = fields.Char('Name', required=True, translate=True)
    code = fields.Char('code', required=True)

    @classmethod
    def __setup__(cls):
        super(EnvironmentalFactorCategory, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class EnvironmentalFactor(ModelSQL, ModelView):
    'Environmental factors restrictions'
    __name__ = 'gnuhealth.environmental_factor'

    name = fields.Char('Environment', required=True, translate=True)
    code = fields.Char('code', required=True)
    category = fields.Many2One(
        'gnuhealth.environmental_factor.category', 'Category')

    @classmethod
    def __setup__(cls):
        super(EnvironmentalFactor, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.code),
                'The code must be unique !'),
        ]


class PatientDisabilityAssessment(ModelSQL, ModelView):
    'Patient Disability Information'
    __name__ = 'gnuhealth.patient.disability_assessment'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)

    assessment_date = fields.Date('Date')

    assessment = fields.Char('Code')

    crutches = fields.Boolean('Crutches')
    wheelchair = fields.Boolean('Wheelchair')

    uxo = fields.Function(fields.Boolean('UXO'), 'get_uxo_status')
    amputee = fields.Function(
        fields.Boolean('Amputee'),
        'get_amputee_status')
    amputee_since = fields.Function(
        fields.Date('Since'),
        'get_amputee_date')

    notes = fields.Text('Notes', help="Extra Information")

    hand_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Hand', sort=False)

    visual_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Visual', sort=False)

    speech_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Speech', sort=False)

    hearing_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Hearing', sort=False)

    cognitive_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Cognitive', sort=False)

    locomotor_function = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'Mobility', sort=False)

    activity_participation = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
    ], 'A & P', sort=False)

    body_functions = fields.One2Many(
        'gnuhealth.body_function.assessment',
        'assessment', 'Body Functions Impairments')

    body_structures = fields.One2Many(
        'gnuhealth.body_structure.assessment',
        'assessment', 'Body Structures Impairments')

    activity_and_participation = fields.One2Many(
        'gnuhealth.activity.assessment',
        'assessment',
        'Activities and Participation Impairments')

    environmental_factor = fields.One2Many(
        'gnuhealth.environment.assessment',
        'assessment',
        'Environmental Factors Barriers')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof',
        help="Authorized health professional")

    def get_uxo_status(self, name):
        return self.patient.uxo

    def get_amputee_status(self, name):
        return self.patient.amputee

    def get_amputee_date(self, name):
        return self.patient.amputee_since

    @staticmethod
    def default_assessment_date():
        return date.today()

    @staticmethod
    def default_healthprof():
        return get_health_professional()


class PatientBodyFunctionAssessment(ModelSQL, ModelView):
    'Body Functions Assessment'
    __name__ = 'gnuhealth.body_function.assessment'

    assessment = fields.Many2One(
        'gnuhealth.patient.disability_assessment',
        'Assessment', required=True)
    body_function = fields.Many2One('gnuhealth.body_function', 'Body Function')
    qualifier = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Severe impairment'),
        ('3', 'Complete impairment'),
        ('8', 'Not specified'),
        ('9', 'Not applicable'),
    ], 'Qualifier', sort=False)


class PatientBodyStructureAssessment(ModelSQL, ModelView):
    'Body Functions Assessment'
    __name__ = 'gnuhealth.body_structure.assessment'

    assessment = fields.Many2One(
        'gnuhealth.patient.disability_assessment',
        'Assessment', required=True)
    body_structure = fields.Many2One(
        'gnuhealth.body_structure', 'Body Structure')
    qualifier1 = fields.Selection([
        (None, ''),
        ('0', 'No impairment'),
        ('1', 'Mild impairment'),
        ('2', 'Moderate impairment'),
        ('3', 'Severe impairment'),
        ('4', 'Complete impairment'),
        ('8', 'Not specified'),
        ('9', 'Not applicable'),
    ], 'Extent', help="Extent of the impairment", sort=False)

    qualifier2 = fields.Selection([
        (None, ''),
        ('0', 'No change in structure'),
        ('1', 'Total absence'),
        ('2', 'Partial absence'),
        ('3', 'Additional part'),
        ('4', 'Aberrant dimensions'),
        ('5', 'Discontinuity'),
        ('6', 'Deviating position'),
        ('7', 'Qualitative changes in structure,'
            ' including accumulation of fluid'),
        ('8', '8 - Not specified'),
        ('9', '9 - Not applicable'),
    ], 'Nature', help="Nature of the change", sort=False)

    body_side = fields.Selection([
        (None, ''),
        ('left', 'Left'),
        ('right', 'Right'),
        ('both', 'Both'),
    ], 'Side', help="Side of the body, if applies", sort=False)


class PatientActivityAndParticipationAsssessment(ModelSQL, ModelView):
    'Activity and Participation Assessment'
    __name__ = 'gnuhealth.activity.assessment'

    assessment = fields.Many2One(
        'gnuhealth.patient.disability_assessment',
        'Assessment', required=True)

    activity_and_participation = fields.Many2One(
        'gnuhealth.activity_and_participation', 'Activity')

    qualifier1 = fields.Selection([
        (None, ''),
        ('0', 'No difficulty'),
        ('1', 'Mild difficulty'),
        ('2', 'Moderate difficulty'),
        ('3', 'Severe difficulty'),
        ('4', 'Complete difficulty'),
        ('8', 'Not specified'),
        ('9', 'Not applicable'),
    ], 'Performance', help="Extent of the difficulty", sort=False)

    qualifier2 = fields.Selection([
        (None, ''),
        ('0', 'No difficulty'),
        ('1', 'Mild difficulty'),
        ('2', 'Moderate difficulty'),
        ('3', 'Severe difficulty'),
        ('4', 'Complete difficulty'),
        ('8', 'Not specified'),
        ('9', 'Not applicable'),
    ], 'Capacity', help="Extent of the dificulty", sort=False)


class PatientEnvironmentalFactorAssessment(ModelSQL, ModelView):
    'Environmental Factors Assessment'
    __name__ = 'gnuhealth.environment.assessment'

    assessment = fields.Many2One(
        'gnuhealth.patient.disability_assessment',
        'Assessment', required=True)

    environmental_factor = fields.Many2One(
        'gnuhealth.environmental_factor', 'Environment')

    qualifier = fields.Selection([
        (None, ''),
        ('0', 'No barriers'),
        ('1', 'Mild barriers'),
        ('2', 'Moderate barriers'),
        ('3', 'Severe barriers'),
        ('4', 'Complete barriers'),
        ('00', 'No facilitator'),
        ('11', 'Mild facilitator'),
        ('22', 'Moderate facilitator'),
        ('33', 'Severe facilitator'),
        ('44', 'Complete facilitator'),
    ], 'Barriers',
        help="Extent of the barriers or facilitators", sort=False)


# Amputation Information
class PatientAmputation(ModelSQL, ModelView):
    'Amputation'
    __name__ = 'gnuhealth.patient.amputation'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)

    amputation_date = fields.Date('Date')

    body_structure = fields.Many2One('gnuhealth.body_structure', 'Structure')

    etiology = fields.Selection([
        (None, ''),
        ('pvd', 'Peripherial Vascular Disease'),
        ('trauma', 'Trauma'),
        ('neoplasia', 'Neoplasia'),
        ('infection', 'Infection'),
        ('congenital', 'Congenital'),
    ], 'Etiology', sort=False)

    limb = fields.Selection([
        (None, ''),
        ('lower', 'lower limb'),
        ('upper', 'upper limb'),
    ], 'Limb', sort=False)

    side = fields.Selection([
        (None, ''),
        ('left', 'left'),
        ('right', 'right'),
        ('both', 'both'),
    ], 'Side', sort=False)

    amputation_level = fields.Selection([
        (None, ''),
        ('sd', 'SD - Shoulder disarticulation'),
        ('th', 'TH - Transhumeral'),
        ('ed', 'ED - Elbow disarticulation'),
        ('tr', 'TR - Transradial'),
        ('wd', 'WD - Wrist disarticulation'),
        ('ph', 'PH'),
        ('hp', 'HD - Hemipelvectomy'),
        ('tf', 'TF - Transfemoral'),
        ('tt', 'TT - Transtibial'),
        ('symes', 'Symes'),
        ('pffd', 'PFFD'),
    ], 'Level', sort=False)

    comments = fields.Char('Comments')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof',
        help="Authorized health professional")

    @staticmethod
    def default_healthprof():
        return get_health_professional()


# Patient Protheses Information
class PatientProthesis(ModelSQL, ModelView):
    'Prothesis'
    __name__ = 'gnuhealth.patient.prothesis'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)

    issue_date = fields.Date('Date')

    prothesis = fields.Many2One(
        'product.product', 'Prothesis', required=True,
        domain=[('is_prothesis', '=', True)],
        help='Prosthetic device')

    comments = fields.Char('Comments')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof',
        help="Prosthetist or authorized health professional")

    @staticmethod
    def default_healthprof():
        return get_health_professional()


class PatientData (ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient'

    disability_assesments = fields.One2Many(
        'gnuhealth.patient.disability_assessment',
        'patient', 'Assessment', readonly=True)



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                       HEALTH OPHTHALMOLOGY package                    #
#                  __init__.py: Package declaration file                #
#########################################################################

from trytond.pool import Pool
from . import health_ophthalmology


def register():
    Pool.register(
        health_ophthalmology.OphthalmologyEvaluation,
        health_ophthalmology.OphthalmologyFindings,
        module='health_ophthalmology', type_='model')



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                      HEALTH SOCIOECONOMICS package                    #
#                  health_socioeconomics.py: main module                #
#########################################################################

from dateutil.relativedelta import relativedelta
from datetime import datetime
from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval, Equal
from trytond.pool import PoolMeta
from trytond.modules.health.core import (get_health_professional,
                                         format_years_months_days)


__all__ = ['Party', 'PatientSESAssessment', 'GnuHealthPatient']


class Party (metaclass=PoolMeta):
    __name__ = 'party.party'

    occupation = fields.Many2One('gnuhealth.occupation', 'Occupation')

    education = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
    ], 'Education', help="Education Level", sort=False)


class PatientSESAssessment(ModelSQL, ModelView):
    'Socioeconomics and Family Functionality Assessment'
    __name__ = 'gnuhealth.ses.assessment'

    STATES = {'readonly': Eval('state') == 'done'}

    patient = fields.Many2One('gnuhealth.patient', 'Patient',
                              required=True, states=STATES)
    gender = fields.Function(fields.Selection([
        (None, ''),
        ('m', 'Male'),
        ('f', 'Female'),
    ], 'Gender'), 'get_patient_gender', searcher='search_patient_gender')

    assessment_date = fields.DateTime('Date', help="Assessment date",
                                      states=STATES)

    computed_age = fields.Function(fields.Char(
        'Age',
        help="Computed patient age at the moment of the evaluation"),
        'patient_age_at_assessment')
    health_professional = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', readonly=True,
        help="Health professional"
    )

    homeless = fields.Boolean(
        "Homeless", help="The person is currently homeless")

    du = fields.Many2One(
        'gnuhealth.du', 'DU', help="Domiciliary Unit",
        states=STATES)

    ses = fields.Selection([
        (None, ''),
        ('0', 'Lower'),
        ('1', 'Lower-middle'),
        ('2', 'Middle'),
        ('3', 'Middle-upper'),
        ('4', 'Higher'),
    ], 'Socioeconomics', help="SES - Socioeconomic Status", sort=False,
        states=STATES)

    ses_str = ses.translated('ses')

    housing = fields.Selection([
        (None, ''),
        ('0', 'Shanty, deficient sanitary conditions'),
        ('1', 'Small, crowded but with good sanitary conditions'),
        ('2', 'Comfortable and good sanitary conditions'),
        ('3', 'Roomy and excellent sanitary conditions'),
        ('4', 'Luxury and excellent sanitary conditions'),
    ], 'Housing conditions',
        help="Housing and sanitary living conditions", sort=False,
        states=STATES)

    occupation = fields.Many2One('gnuhealth.occupation', 'Occupation',
                                 states=STATES)

    income = fields.Selection([
        (None, ''),
        ('l', 'Low'),
        ('m', 'Medium'),
        ('h', 'High'),
    ], 'Income', sort=False, states=STATES)

    fam_apgar_help = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
    ], 'Help from family',
        help="Is the patient satisfied with the level of help coming from "
        "the family when there is a problem ?", sort=False,
        states=STATES)

    fam_apgar_discussion = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
    ], 'Problems discussion',
        help="Is the patient satisfied with the level talking over the "
        "problems as family ?", sort=False, states=STATES)

    fam_apgar_decisions = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
    ], 'Decision making',
        help="Is the patient satisfied with the level of making important "
        "decisions as a group ?", sort=False, states=STATES)

    fam_apgar_timesharing = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
    ], 'Time sharing',
        help="Is the patient satisfied with the level of time that they "
        "spend together ?", sort=False, states=STATES)

    fam_apgar_affection = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
    ], 'Family affection',
        help="Is the patient satisfied with the level of affection coming "
        "from the family ?", sort=False, states=STATES)

    fam_apgar_score = fields.Integer(
        'Score',
        help="Total Family APGAR \n"
        "7 - 10 : Functional Family \n"
        "4 - 6  : Some level of disfunction \n"
        "0 - 3  : Severe disfunctional family \n",
        states=STATES)

    education = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
    ], 'Education Level', help="Education Level", sort=False,
        states=STATES)

    notes = fields.Text('Notes', states=STATES)

    state = fields.Selection([
        (None, ''),
        ('in_progress', 'In progress'),
        ('done', 'Done'),
    ], 'State', readonly=True, sort=False)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={'invisible': Equal(Eval('state'), 'in_progress')},
        help="Health Professional that finished the patient evaluation")

    @fields.depends('fam_apgar_help', 'fam_apgar_timesharing',
                    'fam_apgar_discussion', 'fam_apgar_decisions',
                    'fam_apgar_affection')
    def on_change_with_fam_apgar_score(self):
        fam_apgar_help = int(self.fam_apgar_help or '0')
        fam_apgar_timesharing = int(self.fam_apgar_timesharing or '0')
        fam_apgar_discussion = int(self.fam_apgar_discussion or '0')
        fam_apgar_decisions = int(self.fam_apgar_decisions or '0')
        fam_apgar_affection = int(self.fam_apgar_affection or '0')
        total = (fam_apgar_help + fam_apgar_timesharing +
                 fam_apgar_discussion + fam_apgar_decisions +
                 fam_apgar_affection)

        return total

    @staticmethod
    def default_assessment_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'in_progress'

    @staticmethod
    def default_health_professional():
        return get_health_professional()

    # Show the gender and age upon entering the patient
    # These two are function fields (don't exist at DB level)
    @fields.depends('patient', '_parent_patient.party')
    def on_change_patient(self):
        if self.patient:
            self.gender = self.patient.gender
            self.computed_age = self.patient.age

        occupation = education = du = housing = None
        if (self.patient and self.patient.party.occupation):
            occupation = self.patient.party.occupation

        if (self.patient and self.patient.party.education):
            education = self.patient.party.education

        if (self.patient and self.patient.party.du):
            du = self.patient.party.du

        if (self.patient and self.patient.party.du):
            housing = self.patient.party.du.housing

        self.occupation = occupation
        self.education = education
        self.du = du
        self.housing = housing

    def get_patient_gender(self, name):
        if self.patient:
            return self.patient.gender

    @classmethod
    def search_patient_gender(cls, name, clause):
        res = []
        value = clause[2]
        res.append(('patient.party.gender', clause[1], value))
        return res

    @classmethod
    @ModelView.button
    def end_assessment(cls, assessments):
        # Change the state of the assessment to "Done"
        signing_hp = get_health_professional()

        cls.write(assessments, {
            'state': 'done',
            'signed_by': signing_hp,
        })

    def patient_age_at_assessment(self, name):
        if (self.patient.party.dob and self.assessment_date):
            rdelta = relativedelta(self.assessment_date.date(),
                                   self.patient.party.dob)
            return format_years_months_days(
                years=rdelta.years,
                months=rdelta.months,
                days=rdelta.days)
        else:
            return None

    @classmethod
    def __setup__(cls):
        super(PatientSESAssessment, cls).__setup__()

        cls._buttons.update({
            'end_assessment': {'invisible': Equal(Eval('state'), 'done')}
        })
        cls._order.insert(0, ('assessment_date', 'DESC'))

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None

    @classmethod
    def search_rec_name(cls, name, clause):
        if clause[1].startswith('!') or clause[1].startswith('not '):
            bool_op = 'AND'
        else:
            bool_op = 'OR'
        return [bool_op,
                ('patient',) + tuple(clause[1:]),
                ]


class GnuHealthPatient(ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient'

    occupation = fields.Function(
        fields.Many2One('gnuhealth.occupation',
                        'Occupation'), 'get_patient_occupation')

    education = fields.Function(fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
    ], 'Education Level', help="Education Level", sort=False),
        'get_patient_education')

    education_str = education.translated('education')

    housing = fields.Function(fields.Selection([
        (None, ''),
        ('0', 'Shanty, deficient sanitary conditions'),
        ('1', 'Small, crowded but with good sanitary conditions'),
        ('2', 'Comfortable and good sanitary conditions'),
        ('3', 'Roomy and excellent sanitary conditions'),
        ('4', 'Luxury and excellent sanitary conditions'),
    ], 'Housing conditions', help="Housing and sanitary living conditions",
        sort=False), 'get_patient_housing')

    housing_str = housing.translated('housing')

    ses = fields.Function(fields.Selection([
        (None, ''),
        ('0', 'Lower'),
        ('1', 'Lower-middle'),
        ('2', 'Middle'),
        ('3', 'Middle-upper'),
        ('4', 'Higher'),
    ], 'SES', help="Current Socioeconomic Status", sort=False),
        'get_patient_ses')

    ses_str = ses.translated('ses')

    ses_assessments = fields.One2Many(
        'gnuhealth.ses.assessment',
        'patient', 'Assessments', readonly=True,
        help="Socioeconomics and Family assessments history")

    hostile_area = fields.Boolean(
        'Hostile Area',
        help="Check if patient lives in a zone"
        "of high hostility (eg, war)")

    single_parent = fields.Boolean('Single parent family')
    domestic_violence = fields.Boolean('Domestic violence')
    working_children = fields.Boolean('Working children')
    teenage_pregnancy = fields.Boolean('Teenage pregnancy')
    sexual_abuse = fields.Boolean('Sexual abuse')
    drug_addiction = fields.Boolean('Drug addiction')
    school_withdrawal = fields.Boolean('School withdrawal')
    prison_past = fields.Boolean('Has been in prison')
    prison_current = fields.Boolean('Currently in prison')
    relative_in_prison = fields.Boolean(
        'Relative in prison',
        help="Check if someone from"
        " the nuclear family - parents / "
        "sibblings  is or has been in prison")

    ses_notes = fields.Text('Extra info')

    # GnuHealth 2.0 . Occupation and Education are now functional fields.
    # Retrives the information from the party model.
    occupation = fields.Function(
        fields.Many2One('gnuhealth.occupation',
                        'Occupation'), 'get_patient_occupation')

    works_at_home = fields.Boolean(
        'Works at home',
        help="Check if the patient works at his / her house")
    hours_outside = fields.Integer(
        'Hours outside home',
        help="Number of hours a day the patient"
        "spend outside the house")

    def get_patient_occupation(self, name):
        if (self.party.occupation):
            return self.party.occupation.id

    def get_patient_education(self, name):
        return self.party.education

    def get_patient_housing(self, name):
        if (self.party.du):
            return self.party.du.housing

    def get_patient_ses(self, name):
        if (self.ses_assessments):
            return self.ses_assessments[0].ses



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2013 Sebastián Marró <smarro@thymbra.com>

# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                     HEALTH IMAGING package                            #
#                 health_imaging.pu: Main module                        #
#########################################################################
from datetime import datetime
from trytond.model import Workflow, ModelView, ModelSQL, fields, Unique
from trytond.pyson import Eval
from trytond.pool import Pool

from trytond.modules.health.core import (
    get_health_professional, compute_age_from_dates)


__all__ = [
    'ImagingTestType',
    'ImagingTest', 'ImagingTestRequest', 'ImagingTestResult']


class ImagingTestType(ModelSQL, ModelView):
    'Medical Imaging Study Type'
    __name__ = 'gnuhealth.imaging.test.type'

    code = fields.Char(
        'Code', required=True,
        help="Suggest use values of DICOM "
        "Modality (0008,0060) tag.")
    name = fields.Char(
        'Name', required=True, translate=True,
        help="Suggest use descriptions of DICOM "
        "Modality (0008,0060) tag.")


class ImagingTest(ModelSQL, ModelView):
    'Medical Imaging Study'
    __name__ = 'gnuhealth.imaging.test'

    code = fields.Char('Code', required=True)
    name = fields.Char('Name', required=True, translate=True)
    test_type = fields.Many2One(
        'gnuhealth.imaging.test.type', 'Type',
        required=True)
    product = fields.Many2One('product.product', 'Product', required=True)

    active = fields.Boolean('Active')

    @staticmethod
    def default_active():
        return True


class ImagingTestRequest(Workflow, ModelSQL, ModelView):
    'Medical Imaging Study Request'
    __name__ = 'gnuhealth.imaging.test.request'

    def get_rec_name(self, name):
        res = ''
        if self.urgent:
            res = '**Urgent**'
        if self.doctor:
            res = f'{res} ({self.doctor.rec_name}) //'
        if self.context:
            res = f'{res} CONTEXT: {self.context.rec_name}'
        return res

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)
    date = fields.DateTime('Date', required=True)
    requested_test = fields.Many2One(
        'gnuhealth.imaging.test', 'Study',
        required=True)
    doctor = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof', required=True)
    state = fields.Selection([
        ('draft', 'Draft'),
        ('requested', 'Requested'),
        ('done', 'Done'),
    ], 'State', readonly=True)

    context = fields.Many2One(
        'gnuhealth.pathology', 'Context',
        help="Health context for this order. It can be a suspected or"
             " existing health condition, a regular health checkup, ...")

    comment = fields.Text('Additional Information')
    request = fields.Char('Order', readonly=True)
    request_line = fields.Char('Order line', readonly=True)
    urgent = fields.Boolean('Urgent')

    @classmethod
    def __setup__(cls):
        super(ImagingTestRequest, cls).__setup__()
        cls._transitions |= set((
            ('draft', 'requested'),
            ('requested', 'done')
        ))
        cls._buttons.update({
            'requested': {
                'invisible': ~Eval('state').in_(['draft']),
            },
            'generate_results': {
                'invisible': ~Eval('state').in_(['requested'])
            }
        })
        cls._order.insert(0, ('date', 'DESC'))
        cls._order.insert(1, ('request', 'DESC'))

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None

    @staticmethod
    def default_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'draft'

    @staticmethod
    def default_doctor():
        return get_health_professional()

    @classmethod
    def generate_code(cls, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'imaging_req_seq', **pattern)
        if sequence:
            return sequence.get()

    @classmethod
    def create(cls, vlist):
        vlist = [x.copy() for x in vlist]
        count = len(vlist)
        num = 1
        for values in vlist:
            if not values.get('request'):
                values['request'] = cls.generate_code()
            if not values.get('request_line'):
                line = f'{values["request"]}-{count:02}-{num:02}'
                values['request_line'] = line
            num = num + 1

        return super(ImagingTestRequest, cls).create(vlist)

    @classmethod
    def copy(cls, tests, default=None):
        if default is None:
            default = {}
        default = default.copy()
        default['request'] = None
        default['request_line'] = None
        default['date'] = cls.default_date()
        return super(ImagingTestRequest, cls).copy(tests, default=default)

    @classmethod
    @ModelView.button
    @Workflow.transition('requested')
    def requested(cls, requests):
        pass

    @classmethod
    @ModelView.button_action('health_imaging.wizard_generate_result')
    def generate_results(cls, requests):
        pass

    @classmethod
    @Workflow.transition('done')
    def done(cls, requests):
        pass


class ImagingTestResult(ModelSQL, ModelView):
    'Medical Imaging Study Result'
    __name__ = 'gnuhealth.imaging.test.result'

    def patient_age_at_evaluation(self, name):
        if (self.patient.party.dob and self.date):
            return compute_age_from_dates(
                self.patient.party.dob, None, None, None, 'age',
                self.date.date())

    patient = fields.Many2One('gnuhealth.patient', 'Patient', readonly=True)
    number = fields.Char('Number', readonly=True)
    date = fields.DateTime('Date', required=True)
    request_date = fields.DateTime('Request Date', readonly=True)
    requested_test = fields.Many2One(
        'gnuhealth.imaging.test', 'Study',
        required=True)
    request = fields.Many2One(
        'gnuhealth.imaging.test.request', 'Request Info',
        readonly=True)
    order = fields.Char(
        'Order', readonly=True,
        help="The order ID containing this particular imaging study")
    doctor = fields.Many2One(
        'gnuhealth.healthprofessional', 'Evaluated by', required=True)

    computed_age = fields.Function(fields.Char(
        'Age',
        help="Computed patient age at the moment of the evaluation"),
        'patient_age_at_evaluation')

    comment = fields.Text('Additional Information')

    report_style = fields.Selection([
        ('default', 'Default'),
        ('no_images', 'No Images'),
        ('no_image_comments', 'No Image Comments')
    ], 'Report Style', sort=False)

    @staticmethod
    def default_report_style():
        return 'default'

    images = fields.One2Many('ir.attachment', 'resource', 'Images')

    # Mostly used in report template.
    def report_has_images(self):
        result = (self.images and self.report_style != 'no_images')
        return result

    # Mostly used in report template.
    def report_has_image_comments(self):
        result = (self.report_style != 'no_image_comments' and
                  (True in [
                      img.description != '' and
                      img.description != 'From GNU Health camera' and
                      img.description is not None for img in self.images]))
        return result

    @classmethod
    def generate_code(cls, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'imaging_test_sequence', **pattern)
        if sequence:
            return sequence.get()

    @classmethod
    def create(cls, vlist):
        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('number'):
                values['number'] = cls.generate_code()
        return super(ImagingTestResult, cls).create(vlist)

    @classmethod
    def search_rec_name(cls, name, clause):
        if clause[1].startswith('!') or clause[1].startswith('not '):
            bool_op = 'AND'
        else:
            bool_op = 'OR'
        return [
            bool_op,
            ('patient',) + tuple(clause[1:]),
            ('number',) + tuple(clause[1:]),
        ]

    @classmethod
    def __setup__(cls):
        super(ImagingTestResult, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('number_uniq', Unique(t, t.number),
             'The study ID code must be unique')
        ]
        cls._order.insert(0, ('date', 'DESC'))

    def get_rec_name(self, name):
        res = f'{self.number} ({self.requested_test.name})'
        return res



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                      HEALTH INPATIENT package                         #
#              __init__.py: Package declaration file                    #
#########################################################################
from trytond.pool import Pool
from . import health_inpatient
from . import wizard
from . import sequences
from . import health


def register():
    Pool.register(
        health.PatientProcedure,
        health_inpatient.DietTherapeutic,
        health_inpatient.InpatientRegistration,
        health_inpatient.BedTransfer,
        health_inpatient.Appointment,
        health_inpatient.PatientEvaluation,
        health_inpatient.ECG,
        health_inpatient.PatientData,
        health_inpatient.InpatientMedication,
        health_inpatient.InpatientMedicationAdminTimes,
        health_inpatient.InpatientMedicationLog,
        health_inpatient.InpatientDiet,
        wizard.wizard_health_inpatient.CreateBedTransferInit,
        health_inpatient.InpatientMeal,
        health_inpatient.InpatientMealOrder,
        health_inpatient.InpatientMealOrderItem,
        health_inpatient.PatientRounding,
        health_inpatient.RoundingProcedure,
        sequences.GnuHealthSequences,
        sequences.InpatientRegistrationSequence,
        sequences.InpatientMealOrderSequence,
        sequences.PatientRoundingSequence,
        module='health_inpatient', type_='model')

    Pool.register(
        wizard.wizard_health_inpatient.CreateBedTransfer,
        wizard.wizard_inpatient_evaluation.CreateInpatientEvaluation,
        module='health_inpatient', type_='wizard')



# SPDX-FileCopyrightText: 2020-2021 National University of Entre Rios (UNER)
#                         School of Engineering
#                         <saludpublica@ingenieria.uner.edu.ar>
# SPDX-FileCopyrightText: 2020 Mario Puntin <mario@silix.com.ar>
# SPDX-FileCopyrightText: 2020-2024 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2020-2024 GNU Solidario <health@gnusolidario.org>

# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                         HEALTH DENTISTRY package                      #
#                procedures_report: Procedures report module            #
#########################################################################

from collections import defaultdict

from trytond.pool import Pool
from trytond.report import Report

__all__ = ['DentistryProcedureReport']


class DentistryProcedureReport(Report):
    __name__ = 'health_dentistry.procedure.report'

    @classmethod
    def digest_treatments(cls, patient, lang=None):
        pool = Pool()
        Procedure = pool.get('gnuhealth.dentistry.procedure')

        result = []
        treatments = patient.dentistry_treatments
        for t in treatments:
            add_notes = True
            notes = t.notes.strip()
            data = defaultdict(list)
            for p in t.procedures:
                data[p.procedure.id].append(p.tooth if p.tooth else '')
            for k, v in data.items():
                procedure = Procedure.browse([k])[0]
                procedure_info = procedure.name + ' (' + ', '.join(v) + ')'
                if notes:
                    notes = notes if add_notes else '"'
                    add_notes = False
                result.append({
                    'treatment_date': t.treatment_date,
                    'state': t.state_string,
                    'procedure': procedure_info,
                    'notes': notes,
                })
        return result

    @classmethod
    def get_context(cls, records, header, data):
        pool = Pool()
        Date = pool.get('ir.date')
        context = super(DentistryProcedureReport, cls).get_context(
            records, header, data)
        context['today'] = Date.today()
        context['digest_treatments'] = cls.digest_treatments

        return context



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.pool import PoolMeta

__all__ = ['PatientProcedure']


class PatientProcedure(metaclass=PoolMeta):
    __name__ = 'gnuhealth.patient.procedure'

    """
    Include patient surgery reference to
    list of allowed procedures
    """

    @classmethod
    def _get_origin(cls):
        return super(PatientProcedure, cls)._get_origin() + [
            'gnuhealth.surgery'
        ]



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from . import wizard_health_insurance


__all__ = ['wizard_health_insurance']



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2011 Cédric Krier <cedric.krier@b2ck.com>

# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                        HEALTH ICU package                             #
#                   health_icu.py: main module                          #
#########################################################################

from trytond.model import ModelView, ModelSQL, fields
from datetime import datetime
from trytond.transaction import Transaction
from trytond.pool import PoolMeta
from trytond.pyson import Eval, Not, Bool, Equal
from .exceptions import PatientAlreadyInICU, PatientAlreadyOnMV
from trytond.i18n import gettext


__all__ = ['InpatientRegistration', 'InpatientIcu', 'Glasgow', 'ApacheII',
           'MechanicalVentilation', 'ChestDrainageAssessment',
           'PatientRounding']


class InpatientRegistration(metaclass=PoolMeta):
    __name__ = 'gnuhealth.inpatient.registration'
    icu = fields.Boolean(
        'ICU', help='Shows if patient was admitted to'
        ' the Intensive Care Unit during the hospitalization period')
    icu_admissions = fields.One2Many(
        'gnuhealth.inpatient.icu',
        'registration', "ICU Admissions")


class InpatientIcu(ModelSQL, ModelView):
    'Patient ICU Information'
    __name__ = 'gnuhealth.inpatient.icu'

    def icu_duration(self, name):
        if self.discharged_from_icu:
            end = self.icu_discharge_date
        else:
            end = datetime.now()
        return end.date() - self.icu_admission_date.date()

    registration = fields.Many2One(
        'gnuhealth.inpatient.registration',
        'Registration Code', required=True)

    admitted = fields.Boolean(
        'Admitted',
        help="Will be set when the patient "
        "is currently admitted at ICU.")

    icu_admission_date = fields.DateTime(
        'ICU Admission',
        help="ICU Admission Date", required=True)
    discharged_from_icu = fields.Boolean('Discharged')
    icu_discharge_date = fields.DateTime('Discharge', states={
        'invisible': Not(Bool(Eval('discharged_from_icu'))),
        'required': Bool(Eval('discharged_from_icu')),
    },
        depends=['discharged_from_icu'])
    icu_stay = fields.Function(fields.TimeDelta('Duration'), 'icu_duration')

    mv_history = fields.One2Many(
        'gnuhealth.icu.ventilation',
        'admission', "Mechanical Ventilation History")

    @classmethod
    def validate(cls, inpatients):
        super(InpatientIcu, cls).validate(inpatients)
        for inpatient in inpatients:
            inpatient.check_patient_admitted_at_icu()

    def check_patient_admitted_at_icu(self):
        # Verify that the patient is not at ICU already
        cursor = Transaction().connection.cursor()
        table = self.__class__.__table__()
        cursor.execute(
            *table.select(
                table.registration, where=(
                    (table.registration == self.registration.id) &
                    (table.admitted))))
        res = cursor.fetchall()
        if len(res) > 1:
            raise PatientAlreadyInICU(
                gettext('health_icu.msg_patient_already_in_icu'))

    @staticmethod
    def default_admitted():
        return True

    @fields.depends('discharged_from_icu')
    def on_change_with_admitted(self):
        # Reset the admission flag when the patient is discharged from ICU
        if self.discharged_from_icu:
            res = False
        else:
            res = True
        return res

    @classmethod
    def __register__(cls, module):
        table_h = cls.__table_handler__(module)

        # Migration from 4.4: rename name to registration
        if (table_h.column_exist('name')
                and not table_h.column_exist('registration')):
            table_h.column_rename('name', 'registration')

        super().__register__(module)
        table_h = cls.__table_handler__(module)


class Glasgow(ModelSQL, ModelView):
    'Glasgow Coma Scale'
    __name__ = 'gnuhealth.icu.glasgow'

    registration = fields.Many2One(
        'gnuhealth.inpatient.registration',
        'Registration Code', required=True)

    evaluation_date = fields.DateTime(
        'Date', help="Date / Time",
        required=True)

    glasgow = fields.Integer(
        'Glasgow',
        help='Level of Consciousness - on Glasgow Coma Scale :  < 9 severe -'
        ' 9-12 Moderate, > 13 minor')
    glasgow_eyes = fields.Selection([
        ('1', '1 : Does not Open Eyes'),
        ('2', '2 : Opens eyes in response to painful stimuli'),
        ('3', '3 : Opens eyes in response to voice'),
        ('4', '4 : Opens eyes spontaneously'),
    ], 'Eyes', sort=False)
    glasgow_verbal = fields.Selection([
        ('1', '1 : Makes no sounds'),
        ('2', '2 : Incomprehensible sounds'),
        ('3', '3 : Utters inappropriate words'),
        ('4', '4 : Confused, disoriented'),
        ('5', '5 : Oriented, converses normally'),
    ], 'Verbal', sort=False)
    glasgow_motor = fields.Selection([
        ('1', '1 : Makes no movement'),
        ('2', '2 : Extension to painful stimuli - decerebrate response -'),
        ('3', '3 : Abnormal flexion to painful stimuli \
            (decorticate response)'),
        ('4', '4 : Flexion / Withdrawal to painful stimuli'),
        ('5', '5 : localizes painful stimuli'),
        ('6', '6 : Obeys commands'),
    ], 'Motor', sort=False)

    @staticmethod
    def default_glasgow_eyes():
        return '4'

    @staticmethod
    def default_glasgow_verbal():
        return '5'

    @staticmethod
    def default_glasgow_motor():
        return '6'

    @staticmethod
    def default_glasgow():
        return 15

    # Default evaluation date
    @staticmethod
    def default_evaluation_date():
        return datetime.now()

    @fields.depends('glasgow_verbal', 'glasgow_motor', 'glasgow_eyes')
    def on_change_with_glasgow(self):
        return int(self.glasgow_motor) + int(self.glasgow_eyes) + \
            int(self.glasgow_verbal)

    # Return the Glasgow Score with each component
    def get_rec_name(self, name):
        if self.registration:
            res = str(self.glasgow) + ': ' + 'E' + self.glasgow_eyes + ' V' + \
                self.glasgow_verbal + ' M' + self.glasgow_motor
        return res

    @classmethod
    def __register__(cls, module):
        table_h = cls.__table_handler__(module)

        # Migration from 4.4: rename name to registration
        if (table_h.column_exist('name')
                and not table_h.column_exist('registration')):
            table_h.column_rename('name', 'registration')

        super().__register__(module)
        table_h = cls.__table_handler__(module)

    @classmethod
    def __setup__(cls):
        super(Glasgow, cls).__setup__()

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None


class ApacheII(ModelSQL, ModelView):
    'Apache II scoring'
    __name__ = 'gnuhealth.icu.apache2'

    registration = fields.Many2One(
        'gnuhealth.inpatient.registration',
        'Registration Code', required=True)
    score_date = fields.DateTime(
        'Date', help="Date of the score",
        required=True)

    age = fields.Integer('Age', help='Patient age in years')
    temperature = fields.Float('Temperature', help='Rectal temperature')
    mean_ap = fields.Integer('MAP', help='Mean Arterial Pressure')
    heart_rate = fields.Integer('Heart Rate')
    respiratory_rate = fields.Integer('Respiratory Rate')
    fio2 = fields.Float('FiO2')
    pao2 = fields.Integer('PaO2')
    paco2 = fields.Integer('PaCO2')
    aado2 = fields.Integer('A-a DO2')

    ph = fields.Float('pH')
    serum_sodium = fields.Integer('Sodium')
    serum_potassium = fields.Float('Potassium')
    serum_creatinine = fields.Float('Creatinine')
    arf = fields.Boolean('ARF', help='Acute Renal Failure')
    wbc = fields.Float(
        'WBC', help="White blood cells x 1000 - if you"
        " want to input 4500 wbc / ml, type in 4.5")
    hematocrit = fields.Float('Hematocrit')
    gcs = fields.Integer(
        'GSC', help='Last Glasgow Coma Scale'
        ' You can use the GSC calculator from the Patient Evaluation Form.')
    chronic_condition = fields.Boolean(
        'Chronic condition',
        help='Organ Failure or immunocompromised patient')
    hospital_admission_type = fields.Selection([
        (None, ''),
        ('me', 'Medical or emergency postoperative'),
        ('el', 'elective postoperative')],
        'Hospital Admission Type', states={
            'invisible': Not(Bool(Eval('chronic_condition'))),
            'required': Bool(Eval('chronic_condition'))}, sort=False)

    apache_score = fields.Integer('Score')

    # Default FiO2 PaO2 and PaCO2 so we do the A-a gradient
    # calculation with non-null values

    @fields.depends('fio2', 'pao2', 'paco2')
    def on_change_with_aado2(self):
        # Calculates the Alveolar-arterial difference
        # based on FiO2, PaCO2 and PaO2 values

        if (self.fio2 and self.paco2 and self.pao2):
            return (713 * self.fio2) - (self.paco2 / 0.8) - self.pao2

    @fields.depends(
        'age', 'temperature', 'mean_ap', 'heart_rate',
        'respiratory_rate', 'fio2', 'pao2', 'aado2', 'ph', 'serum_sodium',
        'serum_potassium', 'serum_creatinine', 'arf', 'wbc', 'hematocrit',
        'gcs', 'chronic_condition', 'hospital_admission_type')
    def on_change_with_apache_score(self):
        # Calculate the APACHE SCORE from the variables in the

        total = 0
        # Age
        if (self.age):
            if (self.age > 44 and self.age < 55):
                total = total + 2
            elif (self.age > 54 and self.age < 65):
                total = total + 3
            elif (self.age > 64 and self.age < 75):
                total = total + 5
            elif (self.age > 74):
                total = total + 6

        # Temperature
        if (self.temperature):
            if ((self.temperature >= 38.5 and self.temperature < 39) or
                    (self.temperature >= 34 and self.temperature < 36)):
                total = total + 1
            elif (self.temperature >= 32 and self.temperature < 34):
                total = total + 2
            elif ((self.temperature >= 30 and self.temperature < 32) or
                    (self.temperature >= 39 and self.temperature < 41)):
                total = total + 3
            elif (self.temperature >= 41 or self.temperature < 30):
                total = total + 4

        # Mean Arterial Pressure (MAP)
        if (self.mean_ap):
            if ((self.mean_ap >= 110 and self.mean_ap < 130) or
                    (self.mean_ap >= 50 and self.mean_ap < 70)):
                total = total + 2
            elif (self.mean_ap >= 130 and self.mean_ap < 160):
                total = total + 3
            elif (self.mean_ap >= 160 or self.mean_ap < 50):
                total = total + 4

        # Heart Rate
        if (self.heart_rate):
            if ((self.heart_rate >= 55 and self.heart_rate < 70) or
                    (self.heart_rate >= 110 and self.heart_rate < 140)):
                total = total + 2
            elif ((self.heart_rate >= 40 and self.heart_rate < 55) or
                    (self.heart_rate >= 140 and self.heart_rate < 180)):
                total = total + 3
            elif (self.heart_rate >= 180 or self.heart_rate < 40):
                total = total + 4

        # Respiratory Rate
        if (self.respiratory_rate):
            if ((self.respiratory_rate >= 10 and self.respiratory_rate < 12) or
                (self.respiratory_rate >= 25 and
                    self.respiratory_rate < 35)):
                total = total + 1
            elif (self.respiratory_rate >= 6 and self.respiratory_rate < 10):
                total = total + 2
            elif (self.respiratory_rate >= 35 and self.respiratory_rate < 50):
                total = total + 3
            elif (self.respiratory_rate >= 50 or self.respiratory_rate < 6):
                total = total + 4

        # FIO2
        if (self.fio2):
            # If Fi02 is greater than 0.5, we measure the AaDO2 gradient
            # Otherwise, we take into account the Pa02 value

            if (self.fio2 >= 0.5):
                if (self.aado2 >= 200 and self.aado2 < 350):
                    total = total + 2

                elif (self.aado2 >= 350 and self.aado2 < 500):
                    total = total + 3

                elif (self.aado2 >= 500):
                    total = total + 4

            else:
                if (self.pao2 >= 61 and self.pao2 < 71):
                    total = total + 1

                elif (self.pao2 >= 55 and self.pao2 < 61):
                    total = total + 3

                elif (self.pao2 < 55):
                    total = total + 4

        # Arterial pH
        if (self.ph):
            if (self.ph >= 7.5 and self.ph < 7.6):
                total = total + 1
            elif (self.ph >= 7.25 and self.ph < 7.33):
                total = total + 2
            elif ((self.ph >= 7.15 and self.ph < 7.25) or
                    (self.ph >= 7.6 and self.ph < 7.7)):
                total = total + 3
            elif (self.ph >= 7.7 or self.ph < 7.15):
                total = total + 4

        # Serum Sodium
        if (self.serum_sodium):
            if (self.serum_sodium >= 150 and self.serum_sodium < 155):
                total = total + 1
            elif ((self.serum_sodium >= 155 and self.serum_sodium < 160) or
                    (self.serum_sodium >= 120 and self.serum_sodium < 130)):
                total = total + 2
            elif ((self.serum_sodium >= 160 and self.serum_sodium < 180) or
                    (self.serum_sodium >= 111 and self.serum_sodium < 120)):
                total = total + 3
            elif (self.serum_sodium >= 180 or self.serum_sodium < 111):
                total = total + 4

        # Serum Potassium
        if (self.serum_potassium):
            if ((self.serum_potassium >= 3 and self.serum_potassium < 3.5) or
                (self.serum_potassium >= 5.5 and
                    self.serum_potassium < 6)):
                total = total + 1
            elif (self.serum_potassium >= 2.5 and self.serum_potassium < 3):
                total = total + 2
            elif (self.serum_potassium >= 6 and self.serum_potassium < 7):
                total = total + 3
            elif (self.serum_potassium >= 7 or self.serum_potassium < 2.5):
                total = total + 4

        # Serum Creatinine
        if (self.serum_creatinine):
            arf_factor = 1
            if (self.arf):
                # We multiply by 2 the score if there is concomitant ARF
                arf_factor = 2
            if ((self.serum_creatinine < 0.6) or
                (self.serum_creatinine >= 1.5 and
                    self.serum_creatinine < 2)):
                total = total + 2 * arf_factor
            elif (self.serum_creatinine >= 2 and self.serum_creatinine < 3.5):
                total = total + 3 * arf_factor
            elif (self.serum_creatinine >= 3.5):
                total = total + 4 * arf_factor

        # Hematocrit
        if (self.hematocrit):
            if (self.hematocrit >= 46 and self.hematocrit < 50):
                total = total + 1
            elif ((self.hematocrit >= 50 and self.hematocrit < 60) or
                    (self.hematocrit >= 20 and self.hematocrit < 30)):
                total = total + 2
            elif (self.hematocrit >= 60 or self.hematocrit < 20):
                total = total + 4

        # WBC ( x 1000 )
        if (self.wbc):
            if (self.wbc >= 15 and self.wbc < 20):
                total = total + 1
            elif ((self.wbc >= 20 and self.wbc < 40) or
                    (self.wbc >= 1 and self.wbc < 3)):
                total = total + 2
            elif (self.wbc >= 40 or self.wbc < 1):
                total = total + 4

        # Immnunocompromised or severe organ failure
        if (self.chronic_condition):
            if (self.hospital_admission_type == 'me'):
                total = total + 5
            else:
                total = total + 2

        return total

    @classmethod
    def __register__(cls, module):
        table_h = cls.__table_handler__(module)

        # Migration from 4.4: rename name to registration
        if (table_h.column_exist('name')
                and not table_h.column_exist('registration')):
            table_h.column_rename('name', 'registration')

        super().__register__(module)
        table_h = cls.__table_handler__(module)


class MechanicalVentilation(ModelSQL, ModelView):
    'Mechanical Ventilation History'
    __name__ = 'gnuhealth.icu.ventilation'

    def mv_duration(self, name):
        start = end = datetime.now()

        if self.mv_start:
            start = self.mv_start

        if self.mv_end:
            end = self.mv_end

        return end.date() - start.date()

    admission = fields.Many2One(
        'gnuhealth.inpatient.icu', 'Patient ICU Admission',
        required=True)

    ventilation = fields.Selection([
        (None, ''),
        ('none', 'None - Maintains Own'),
        ('nppv', 'Non-Invasive Positive Pressure'),
        ('ett', 'ETT'),
        ('tracheostomy', 'Tracheostomy')],
        'Type',
        help="NPPV = Non-Invasive Positive "
        "Pressure Ventilation, BiPAP-CPAP \n"
        "ETT - Endotracheal Tube", sort=False)

    ett_size = fields.Integer('ETT Size', states={
        'invisible': Not(Equal(Eval('ventilation'), 'ett'))})

    tracheostomy_size = fields.Integer('Tracheostomy size', states={
        'invisible': Not(Equal(Eval('ventilation'), 'tracheostomy'))})

    mv_start = fields.DateTime(
        'From', help="Start of Mechanical Ventilation",
        required=True)
    mv_end = fields.DateTime(
        'To', help="End of Mechanical Ventilation",
        states={
            'invisible': Bool(Eval('current_mv')),
            'required': Not(Bool(Eval('current_mv'))),
        },
        depends=['current_mv'])
    mv_period = fields.Function(fields.TimeDelta('Duration'), 'mv_duration')
    current_mv = fields.Boolean('Current')
    remarks = fields.Char('Remarks')

    @classmethod
    def validate(cls, inpatients):
        super(MechanicalVentilation, cls).validate(inpatients)
        for inpatient in inpatients:
            inpatient.check_patient_current_mv()

    def check_patient_current_mv(self):
        # Check for only one current mechanical ventilation on patient
        cursor = Transaction().connection.cursor()
        table = self.__class__.__table__()
        cursor.execute(*table.select(
            table.admission, where=(
                (table.admission == self.admission.id)
                & (table.current_mv))))
        res = cursor.fetchall()
        if len(res) > 1:
            raise PatientAlreadyOnMV(
                gettext('health_icu.msg_patient_already_on_mv'))

    @staticmethod
    def default_current_mv():
        return True

    @classmethod
    def __register__(cls, module):
        table_h = cls.__table_handler__(module)

        # Migration from 4.4: rename name to registration
        if (table_h.column_exist('name')
                and not table_h.column_exist('admission')):
            table_h.column_rename('name', 'admission')

        super().__register__(module)
        table_h = cls.__table_handler__(module)


class ChestDrainageAssessment(ModelSQL, ModelView):
    'Chest Drainage Asessment'
    __name__ = 'gnuhealth.icu.chest_drainage'

    rounding = fields.Many2One(
        'gnuhealth.patient.rounding', 'Rounding',
        required=True)
    location = fields.Selection([
        (None, ''),
        ('rl', 'Right Pleura'),
        ('ll', 'Left Pleura'),
        ('mediastinum', 'Mediastinum')],
        'Location', sort=False)
    fluid_aspect = fields.Selection([
        (None, ''),
        ('serous', 'Serous'),
        ('bloody', 'Bloody'),
        ('chylous', 'Chylous'),
        ('purulent', 'Purulent')],
        'Aspect', sort=False)
    suction = fields.Boolean('Suction')
    suction_pressure = fields.Integer('cm H2O', states={
        'invisible': Not(Bool(Eval('suction'))),
        'required': Bool(Eval('suction')),
    },
        depends=['suction'])
    oscillation = fields.Boolean('Oscillation')
    air_leak = fields.Boolean('Air Leak')
    fluid_volume = fields.Integer('Volume')
    remarks = fields.Char('Remarks')

    @classmethod
    def __register__(cls, module):
        table_h = cls.__table_handler__(module)

        # Migration from 4.4: rename name to rounding
        if (table_h.column_exist('name')
                and not table_h.column_exist('rounding')):
            table_h.column_rename('name', 'rounding')

        super().__register__(module)
        table_h = cls.__table_handler__(module)


class PatientRounding(metaclass=PoolMeta):
    # Nursing Rounding for ICU
    # Inherit and append to the existing model the new functionality for ICU

    __name__ = 'gnuhealth.patient.rounding'

    STATES = {'readonly': Eval('state') == 'done'}

    icu_patient = fields.Boolean(
        'ICU', help='Check this box if this is'
        'an Intensive Care Unit rounding.', states=STATES)
    # Neurological assesment
    gcs = fields.Many2One(
        'gnuhealth.icu.glasgow', 'GCS',
        domain=[
            ('registration', '=', Eval('registration'))],
        depends=['registration'], states=STATES)

    pupil_dilation = fields.Selection([
        ('normal', 'Normal'),
        ('miosis', 'Miosis'),
        ('mydriasis', 'Mydriasis')],
        'Pupil Dilation', sort=False, states=STATES)

    left_pupil = fields.Integer(
        'L', help="size in mm of left pupil",
        states=STATES)
    right_pupil = fields.Integer(
        'R', help="size in mm of right pupil",
        states=STATES)

    anisocoria = fields.Boolean('Anisocoria', states=STATES)

    pupillary_reactivity = fields.Selection([
        (None, ''),
        ('brisk', 'Brisk'),
        ('sluggish', 'Sluggish'),
        ('nonreactive', 'Nonreactive')],
        'Pupillary Reactivity', sort=False, states=STATES)

    pupil_consensual_resp = fields.Boolean(
        'Consensual Response',
        help="Pupillary Consensual Response", states=STATES)

    # Respiratory assesment
    # Mechanical ventilation information is on the patient ICU general info

    respiration_type = fields.Selection([
        (None, ''),
        ('regular', 'Regular'),
        ('deep', 'Deep'),
        ('shallow', 'Shallow'),
        ('labored', 'Labored'),
        ('intercostal', 'Intercostal')],
        'Respiration', sort=False, states=STATES)

    oxygen_mask = fields.Boolean('Oxygen Mask', states=STATES)
    fio2 = fields.Integer('FiO2', states=STATES)

    peep = fields.Boolean('PEEP', states=STATES)

    peep_pressure = fields.Integer('cm H2O', help="Pressure", states={
        'invisible': Not(Bool(Eval('peep'))),
        'required': Bool(Eval('peep')),
        'readonly': Eval('state') == 'done',
    },
        depends=['peep'])

    sce = fields.Boolean(
        'SCE', help="Subcutaneous Emphysema", states=STATES)
    lips_lesion = fields.Boolean('Lips lesion', states=STATES)
    oral_mucosa_lesion = fields.Boolean('Oral mucosa lesion', states=STATES)

    # Chest expansion characteristics
    chest_expansion = fields.Selection([
        (None, ''),
        ('symmetric', 'Symmetrical'),
        ('asymmetric', 'Asymmetrical')],
        'Expansion', sort=False, states=STATES)
    paradoxical_expansion = fields.Boolean(
        'Paradoxical',
        help="Paradoxical Chest Expansion", states=STATES)
    tracheal_tug = fields.Boolean('Tracheal Tug', states=STATES)

    # Trachea position
    trachea_alignment = fields.Selection([
        (None, ''),
        ('midline', 'Midline'),
        ('right', 'Deviated right'),
        ('left', 'Deviated left')],
        'Tracheal alignment', sort=False, states=STATES)

    # Chest Drainages
    chest_drainages = fields.One2Many(
        'gnuhealth.icu.chest_drainage',
        'rounding', "Drainages", states=STATES)

    # Chest X-Ray
    xray = fields.Binary('Xray', states=STATES)

    # Cardiovascular assessment

    ecg = fields.Many2One(
        'gnuhealth.patient.ecg', 'Inpatient ECG',
        domain=[
            ('inpatient_registration_code', '=', Eval('registration'))],
        depends=['registration'], states=STATES)

    venous_access = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('central', 'Central catheter'),
        ('peripheral', 'Peripheral')],
        'Venous Access', sort=False, states=STATES)

    swan_ganz = fields.Boolean(
        'Swan Ganz',
        help="Pulmonary Artery Catheterization - PAC -", states=STATES)

    arterial_access = fields.Boolean('Arterial Access', states=STATES)

    dialysis = fields.Boolean('Dialysis', states=STATES)

    edema = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('peripheral', 'Peripheral'),
        ('anasarca', 'Anasarca')],
        'Edema', sort=False, states=STATES)

    # Blood & Skin
    bacteremia = fields.Boolean('Bacteremia', states=STATES)
    ssi = fields.Boolean('Surgery Site Infection', states=STATES)
    wound_dehiscence = fields.Boolean('Wound Dehiscence', states=STATES)
    cellulitis = fields.Boolean('Cellulitis', states=STATES)
    necrotizing_fasciitis = fields.Boolean(
        'Necrotizing fasciitis',
        states=STATES)

    # Abdomen & Digestive
    vomiting = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('vomiting', 'Vomiting'),
        ('hematemesis', 'Hematemesis')],
        'Vomiting', sort=False, states=STATES)

    bowel_sounds = fields.Selection([
        (None, ''),
        ('normal', 'Normal'),
        ('increased', 'Increased'),
        ('decreased', 'Decreased'),
        ('absent', 'Absent')],
        'Bowel Sounds', sort=False, states=STATES)

    stools = fields.Selection([
        (None, ''),
        ('normal', 'Normal'),
        ('constipation', 'Constipation'),
        ('diarrhea', 'Diarrhea'),
        ('melena', 'Melena')],
        'Stools', sort=False, states=STATES)

    peritonitis = fields.Boolean('Peritonitis signs', states=STATES)

    @fields.depends('left_pupil', 'right_pupil')
    def on_change_with_anisocoria(self):
        if (self.left_pupil == self.right_pupil):
            return False
        else:
            return True

    @staticmethod
    def default_pupil_dilation():
        return 'normal'



# SPDX-FileCopyrightText: 2009-2013 Bertrand Chenal
# SPDX-FileCopyrightText: 2009-2016 B2CK
# SPDX-FileCopyrightText: 2009-2016 Cédric Krier
# SPDX-FileCopyrightText: 2009-2016 Tryton Foundation <info@tryton.org>
# SPDX-FileCopyrightText: 2016-2024 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2016-2024 Luis Falcón <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""GNU Health Calendar for WebDAV3

The package is the continuation of the CalDAV functionality
for the discontinued Tryton package.

It has been ported to Python 3 and GNU Health.

It contains the models to use Calendars in GNU Health HMIS.



"""

from trytond.pool import Pool
from . import webdav
from . import calendar_
from . import res


def register():
    Pool.register(
        webdav.Collection,
        calendar_.Calendar,
        calendar_.ReadUser,
        calendar_.WriteUser,
        calendar_.Category,
        calendar_.Location,
        calendar_.Event,
        calendar_.EventCategory,
        calendar_.EventAlarm,
        calendar_.EventAttendee,
        calendar_.EventRDate,
        calendar_.EventExDate,
        calendar_.EventRRule,
        calendar_.EventExRule,
        res.User,
        module='health_caldav', type_='model')



# Copyright (C) 2008-2025 Luis Falcon <falcon@gnuhealth.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# import pytz
# from datetime import datetime
from trytond.pool import Pool
# from trytond.transaction import Transaction
from trytond.report import Report

__all__ = ['PatientEvaluationReport']


class PatientEvaluationReport(Report):
    __name__ = 'patient.evaluation'

    @classmethod
    def module_status(cls, module_name):
        Module = Pool().get('ir.module')
        module = Module.search([('name', '=', module_name)], limit=1)
        if (module and module[0].state == 'activated'):
            return True
        else:
            return False

    @classmethod
    def get_context(cls, records, header, data):
        context = super(
            PatientEvaluationReport, cls).get_context(records, header, data)

        # Check if the socioecomics package is installed
        if (cls.module_status('health_socioeconomics')):
            context['socioeconomicsmod'] = True
        else:
            context['socioeconomicsmod'] = False

        # Check if the surgery package is installed
        if (cls.module_status('health_surgery')):
            context['surgerymod'] = True
        else:
            context['surgerymod'] = False

        # Check if the genetics package is installed
        if (cls.module_status('health_genetics')):
            context['geneticsmod'] = True
        else:
            context['geneticsmod'] = False

        # Check if the lifestyle package is installed
        if (cls.module_status('health_lifestyle')):
            context['lifestylemod'] = True
        else:
            context['lifestylemod'] = False

        # Check if the crypto package is installed
        if (cls.module_status('health_crypto')):
            context['cryptomod'] = True
        else:
            context['cryptomod'] = False

        return context



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                           HEALTH package                              #
#                     immunization_status_report.py                     #
#########################################################################

from trytond.report import Report
from trytond.pool import Pool
from trytond.modules.health.core import parse_compute_age

__all__ = ['ImmunizationStatusReport']


class ImmunizationStatusReport(Report):
    __name__ = 'gnuhealth.immunization_status_report'

    @classmethod
    def get_context(cls, records, header, data):
        Sched = Pool().get('gnuhealth.immunization_schedule')
        Patient = Pool().get('gnuhealth.patient')
        patient = Patient(data['patient_id'])

        context = super(ImmunizationStatusReport, cls).get_context(
            records, header, data)

        context['patient'] = patient
        sched = Sched(data['immunization_schedule_id'])

        context['immunization_schedule'] = sched

        immunizations_to_check = \
            cls.get_immunizations_for_age(patient, sched)

        immunization_status = \
            cls.verify_status(immunizations_to_check)

        context['immunization_status'] = immunization_status

        return context

    @classmethod
    def get_immunizations_for_age(cls, patient, immunization_schedule):

        immunizations_for_age = []

        for vaccine in immunization_schedule.vaccines:

            for dose in vaccine.doses:
                dose_number, dose_age, age_unit, age_unit_str = \
                    dose.dose_number, dose.age_dose, \
                    dose.age_unit, dose.age_unit_str

                # Age of the person in years, months, weeks and days.
                y, m, d = parse_compute_age(patient.age)
                pdays = (y * 365) + (m * 365 / 12) + d
                pyears = pdays / 365
                pmonths = pdays / (365 / 12)
                pweeks = pdays / 7

                if ((age_unit == 'days' and pdays >= dose_age) or
                    (age_unit == 'weeks' and pweeks >= dose_age) or
                    (age_unit == 'months' and pmonths >= dose_age) or
                        (age_unit == 'years' and pyears >= dose_age)):
                    immunization_info = {
                        'patient': patient,
                        'vaccine': vaccine,
                        'dose': dose_number,
                        'dose_age': dose_age,
                        'age_unit': age_unit,
                        'age_unit_str': age_unit_str,
                        'status': None}

                    # Add to the list of this person immunization check
                    immunizations_for_age.append(immunization_info)

        return immunizations_for_age

    @classmethod
    def verify_status(cls, immunizations_to_check):
        Vaccination = Pool().get('gnuhealth.vaccination')

        result = []
        for immunization in immunizations_to_check:
            immunization['status'] = "missing"
            res = Vaccination.search_count([
                ('patient', '=', immunization['patient']),
                ('dose', '=', immunization['dose']),
                ('vaccine.product', '=',
                 immunization['vaccine'].vaccine.product),
            ])

            if res:
                immunization['status'] = 'ok'

            result.append(immunization)

        return result



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2013 Sebastian Marro <smarro@thymbra.com>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                         HEALTH REPORTING package                      #
#                       health_stock.py: main module                    #
#########################################################################
from datetime import datetime
from trytond.model import Workflow, ModelView, ModelSQL, fields
from trytond.pyson import If, Or, Eval, Bool
from trytond.exceptions import UserError
from trytond.pool import Pool, PoolMeta
from trytond.modules.health.core import get_health_professional, \
    get_institution_currency


__all__ = ['Move',
           'PatientAmbulatoryCare', 'PatientAmbulatoryCareMedicament',
           'PatientAmbulatoryCareMedicalSupply'
           ]

_STATES = {
    'readonly': Eval('state') == 'done',
}
_DEPENDS = ['state']


class Move(metaclass=PoolMeta):
    __name__ = 'stock.move'

    @classmethod
    def _get_origin(cls):
        return super(Move, cls)._get_origin() + [
            'gnuhealth.patient.ambulatory_care',
        ]


class PatientAmbulatoryCare(Workflow, metaclass=PoolMeta):
    'Patient Ambulatory Care'
    __name__ = 'gnuhealth.patient.ambulatory_care'

    care_location = fields.Many2One(
        'stock.location', 'Care Location',
        domain=[('type', '=', 'storage')],
        states={
            'required': If(
                Or(Bool(Eval('medicaments')),
                    Bool(Eval('medical_supplies'))), True, False),
            'readonly': Eval('state') == 'done',
        }, depends=['state', 'medicaments'])
    medicaments = fields.One2Many(
        'gnuhealth.patient.ambulatory_care.medicament', 'ambcare',
        'Medicaments', states=_STATES, depends=_DEPENDS)
    medical_supplies = fields.One2Many(
        'gnuhealth.patient.ambulatory_care.medical_supply', 'ambcare',
        'Medical Supplies', states=_STATES, depends=_DEPENDS)

    moves = fields.One2Many(
        'stock.move', 'origin', 'Stock Moves',
        readonly=True)

    @classmethod
    def __setup__(cls):
        super(PatientAmbulatoryCare, cls).__setup__()
        cls._transitions |= set((
            ('draft', 'done'),
        ))
        cls._buttons.update({
            'done': {
                'invisible': ~Eval('state').in_(['draft']),
            }})

    @classmethod
    def copy(cls, ambulatory_cares, default=None):
        if default is None:
            default = {}
        default = default.copy()
        default['moves'] = None
        return super(PatientAmbulatoryCare, cls).copy(
            ambulatory_cares, default=default)

    @classmethod
    @ModelView.button
    @Workflow.transition('done')
    def done(cls, ambulatory_cares):
        lines_to_ship = {}
        medicaments_to_ship = []
        supplies_to_ship = []

        signing_hp = get_health_professional()

        for ambulatory in ambulatory_cares:
            for medicament in ambulatory.medicaments:
                medicaments_to_ship.append(medicament)

            for medical_supply in ambulatory.medical_supplies:
                supplies_to_ship.append(medical_supply)

        lines_to_ship['medicaments'] = medicaments_to_ship
        lines_to_ship['supplies'] = supplies_to_ship

        cls.create_stock_moves(ambulatory_cares, lines_to_ship)

        cls.write(ambulatory_cares, {
            'signed_by': signing_hp,
            'session_end': datetime.now()
        })

    @classmethod
    def create_stock_moves(cls, ambulatory_cares, lines):
        pool = Pool()
        Move = pool.get('stock.move')
        Date = pool.get('ir.date')
        moves = []
        for ambulatory in ambulatory_cares:
            for medicament in lines['medicaments']:
                move_info = {}
                move_info['origin'] = str(ambulatory)
                move_info['product'] = medicament.medicament.product.id
                move_info['unit'] = \
                    medicament.medicament.product.default_uom.id
                move_info['currency'] = get_institution_currency()
                move_info['quantity'] = medicament.quantity
                move_info['from_location'] = ambulatory.care_location.id
                move_info['to_location'] = \
                    ambulatory.patient.party.customer_location.id
                move_info['unit_price'] = \
                    medicament.medicament.product.list_price
                move_info['cost_price'] = \
                    medicament.medicament.product.cost_price
                if medicament.lot:
                    if medicament.lot.expiration_date and \
                            medicament.lot.expiration_date < Date.today():
                        raise UserError('Expired medicaments')
                    move_info['lot'] = medicament.lot.id
                moves.append(move_info)

            for medical_supply in lines['supplies']:
                move_info = {}
                move_info['origin'] = str(ambulatory)
                move_info['product'] = medical_supply.product.id
                move_info['unit'] = medical_supply.product.default_uom.id
                move_info['currency'] = get_institution_currency()
                move_info['quantity'] = medical_supply.quantity
                move_info['from_location'] = ambulatory.care_location.id
                move_info['to_location'] = \
                    ambulatory.patient.party.customer_location.id
                move_info['unit_price'] = medical_supply.product.list_price
                if medical_supply.lot:
                    if medical_supply.lot.expiration_date \
                            and medical_supply.lot.expiration_date \
                            < Date.today():
                        raise UserError('Expired supplies')
                    move_info['lot'] = medical_supply.lot.id
                moves.append(move_info)

        new_moves = Move.create(moves)
        Move.write(new_moves, {
            'state': 'done',
            'effective_date': Date.today(),
        })

        return True


class PatientAmbulatoryCareMedicament(ModelSQL, ModelView):
    'Patient Ambulatory Care Medicament'
    __name__ = 'gnuhealth.patient.ambulatory_care.medicament'

    ambcare = fields.Many2One(
        'gnuhealth.patient.ambulatory_care',
        'Ambulatory ID')
    medicament = fields.Many2One(
        'gnuhealth.medicament', 'Medicament',
        required=True)
    product = fields.Many2One('product.product', 'Product')
    quantity = fields.Integer('Quantity')
    short_comment = fields.Char(
        'Comment',
        help='Short comment on the specific drug')
    lot = fields.Many2One(
        'stock.lot', 'Lot', depends=['product'],
        domain=[('product', '=', Eval('product'))])

    @staticmethod
    def default_quantity():
        return 1

    @fields.depends('medicament')
    def on_change_medicament(self):
        if self.medicament:
            self.product = self.medicament.product.id

        else:
            self.product = None


class PatientAmbulatoryCareMedicalSupply(ModelSQL, ModelView):
    'Patient Ambulatory Care Medical Supply'
    __name__ = 'gnuhealth.patient.ambulatory_care.medical_supply'

    ambcare = fields.Many2One(
        'gnuhealth.patient.ambulatory_care',
        'Ambulatory ID')
    product = fields.Many2One(
        'product.product', 'Medical Supply',
        domain=[('is_medical_supply', '=', True)], required=True)
    quantity = fields.Integer('Quantity')
    short_comment = fields.Char(
        'Comment',
        help='Short comment on the specific drug')
    lot = fields.Many2One(
        'stock.lot', 'Lot', depends=['product'],
        domain=[
            ('product', '=', Eval('product')),
        ])

    @staticmethod
    def default_quantity():
        return 1



# SPDX-FileCopyrightText: 2009-2013 Bertrand Chenal
# SPDX-FileCopyrightText: 2009-2016 B2CK
# SPDX-FileCopyrightText: 2009-2016 Cédric Krier
# SPDX-FileCopyrightText: 2016-2024 Luis Falcon <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class CalendarTestCase(ModuleTestCase):
    'Test Calendar module'
    module = 'health_caldav'


del ModuleTestCase


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        CalendarTestCase))
    return suite



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                           HEALTH ARCHIVES package                     #
#                      __init__.py  test initialization                 #
#########################################################################
from .test_health_archives import suite


__all__ = ['suite']



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011  Adrián Bernardi, Mario Puntin (health_invoice)
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                         HEALTH SERVICES PACKAGE                       #
#                     health_services.py: Main module                   #
#########################################################################
import datetime
from trytond.model import ModelView, ModelSQL, fields, Unique
from trytond.transaction import Transaction
from trytond.pyson import Eval, Equal
from trytond.pool import Pool, PoolMeta
from trytond.i18n import gettext
from trytond.modules.health.core import get_institution

from .exceptions import (
    ServiceAlreadyInvoiced, NoServiceAssociated, NoProductAssociated,
    ServiceHasBeenUpdated)


__all__ = ['HealthService', 'HealthServiceLine', 'PatientPrescriptionOrder']


class HealthService(ModelSQL, ModelView):
    'Health Service'
    __name__ = 'gnuhealth.health_service'

    STATES = {'readonly': Eval('state') == 'invoiced'}

    name = fields.Char('ID', readonly=True)
    desc = fields.Char('Description')
    patient = fields.Many2One(
        'gnuhealth.patient',
        'Patient', required=True,
        states=STATES)
    institution = fields.Many2One('gnuhealth.institution', 'Institution')
    company = fields.Many2One('company.company', 'Company')

    service_date = fields.Date('Date')
    service_line = fields.One2Many(
        'gnuhealth.health_service.line',
        'service', 'Service Line', help="Service Line")
    state = fields.Selection([
        ('draft', 'Draft'),
        ('invoiced', 'Invoiced'),
    ], 'State', readonly=True)
    invoice_to = fields.Many2One('party.party', 'Invoice to')

    @classmethod
    def __setup__(cls):
        super(HealthService, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('name_unique', Unique(t, t.name),
                'The Service ID must be unique'),
        ]
        cls._buttons.update({
            'button_set_to_draft': {
                'invisible': Equal(Eval('state'), 'draft')}
        })

        cls._order.insert(0, ('state', 'ASC'))
        cls._order.insert(1, ('name', 'DESC'))

    @staticmethod
    def default_state():
        return 'draft'

    @staticmethod
    def default_company():
        return Transaction().context.get('company')

    @staticmethod
    def default_service_date():
        return datetime.date.today()

    @staticmethod
    def default_institution():
        return get_institution()

    @classmethod
    @ModelView.button
    def button_set_to_draft(cls, services):
        cls.write(services, {'state': 'draft'})

    @classmethod
    def generate_code(cls, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'health_service_sequence', **pattern)
        if sequence:
            return sequence.get()

    @classmethod
    def create(cls, vlist):
        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                values['name'] = cls.generate_code()
        return super(HealthService, cls).create(vlist)


class HealthServiceLine(ModelSQL, ModelView):
    'Health Service'
    __name__ = 'gnuhealth.health_service.line'

    service = fields.Many2One(
        'gnuhealth.health_service', 'Service',
        readonly=True)
    desc = fields.Char('Description', required=True)
    appointment = fields.Many2One(
        'gnuhealth.appointment', 'Appointment',
        help='Enter or select the date / ID of the appointment related to'
        ' this evaluation')
    to_invoice = fields.Boolean('Invoice')
    product = fields.Many2One('product.product', 'Product', required=True)
    qty = fields.Integer('Qty')
    from_date = fields.Date('From')
    to_date = fields.Date('To')
    action_required = fields.Boolean(
        'Action required', help='This optional field'
        ' is used in the context of validation'
        ' on the service line. Mark it if there'
        ' is any administrative or other type'
        ' that needs action')

    remarks = fields.Char('Remarks')

    @staticmethod
    def default_qty():
        return 1

    @staticmethod
    def default_to_invoice():
        return True

    @fields.depends('product', 'desc')
    def on_change_product(self, name=None):
        if self.product:
            self.desc = self.product.name

    @classmethod
    def validate(cls, services):
        super(HealthServiceLine, cls).validate(services)
        for service in services:
            service.validate_invoice_status()

    def validate_invoice_status(self):
        if (self.service):
            if (self.service.state == 'invoiced'):
                raise ServiceAlreadyInvoiced(
                    gettext('health_services.msg_service_already_invoiced'))

    def get_rec_name(self, name):
        if self.service:
            return f'{self.desc} ({self.service.name})'


# Add Prescription order charges to service model
class PatientPrescriptionOrder(metaclass=PoolMeta):
    __name__ = 'gnuhealth.prescription.order'

    service = fields.Many2One(
        'gnuhealth.health_service', 'Service',
        domain=[('patient', '=', Eval('patient'))], depends=['patient'],
        states={'readonly': Equal(Eval('state'), 'done')},
        help="Service document associated to this prescription")

    service_updated = fields.Selection((
        ('yes', 'Yes'),
        ('no', 'No'),
        ('unknown', 'Unknown'),
    ), 'Service updated', sort=False)

    @classmethod
    def default_service_updated(self):
        return 'unknown'

    @classmethod
    def __setup__(cls):
        super(PatientPrescriptionOrder, cls).__setup__()
        cls._buttons.update({
            'update_service': {
                'readonly': Equal(Eval('state'), 'done'),
            },
        })

    @classmethod
    @ModelView.button
    def update_service(cls, prescriptions):
        pool = Pool()
        HealthService = pool.get('gnuhealth.health_service')

        hservice = []
        prescription = prescriptions[0]

        if not prescription.service:
            raise NoServiceAssociated(
                gettext('health_services.msg_no_service_associated'))

        if prescription.service_updated == 'yes':
            raise ServiceHasBeenUpdated(
                gettext('health_services.msg_service_has_been_updated'))

        service_data = {}
        service_lines = []

        # Add the prescription lines to the service document

        for line in prescription.prescription_line:
            service_lines.append(('create', [{
                'product': line.medicament.product.id,
                'desc': 'Prescription Line',
                'qty': line.quantity
            }]))

        hservice.append(prescription.service)

        description = "Services including " + \
            prescription.prescription_id

        service_data['desc'] = description
        service_data['service_line'] = service_lines

        HealthService.write(hservice, service_data)

        cls.write(prescriptions, {'service_updated': 'yes'})


# Include  Patient Evaluation service
class PatientEvaluation(metaclass=PoolMeta):
    __name__ = 'gnuhealth.patient.evaluation'

    service = fields.Many2One(
        'gnuhealth.health_service', 'Service',
        domain=[('patient', '=', Eval('patient'))], depends=['patient'],
        states={'readonly': Equal(Eval('state'), 'done')},
        help="Service document associated to this evaluation")

    service_updated = fields.Selection((
        ('yes', 'Yes'),
        ('no', 'No'),
        ('unknown', 'Unknown'),
    ), 'Service updated', sort=False)

    product = fields.Many2One('product.product', 'Product')

    @classmethod
    def default_service_updated(self):
        return 'unknown'

    @classmethod
    def __setup__(cls):
        super(PatientEvaluation, cls).__setup__()
        cls._buttons.update({
            'update_service': {
                'readonly': Equal(Eval('state'), 'done'),
            },
        })

    @classmethod
    @ModelView.button
    def update_service(cls, evaluations):
        pool = Pool()
        HealthService = pool.get('gnuhealth.health_service')

        hservice = []
        evaluation = evaluations[0]

        if not evaluation.service:
            raise NoServiceAssociated(
                gettext('health_services.msg_no_service_associated'))

        if not evaluation.product:
            raise NoProductAssociated(
                gettext('health_services.msg_no_product_associated'))

        if evaluation.service_updated == 'yes':
            raise ServiceHasBeenUpdated(
                gettext('health_services.msg_service_has_been_updated'))

        service_data = {}
        service_lines = []

        # Add the evaluation to the service document line

        service_lines.append(('create', [{
            'product': evaluation.product.id,
            'desc': 'Medical evaluation services',
            'qty': 1
        }]))

        hservice.append(evaluation.service)

        description = "Medical evaluation services"
        service_data['desc'] = description
        service_data['service_line'] = service_lines

        HealthService.write(hservice, service_data)

        cls.write(evaluations, {'service_updated': 'yes'})
